%{
#include "gbdl.h"

char * interpret_panic_code(long code) {
  char *origmessage = new char[75];
  char *message = origmessage+10;

  strcpy(origmessage,"SGB Error:");
  

  if(code < 30) {
    // [-1,29]
    if(code < 11) {
      // [-1,10]
      if(code < 10) {
	// [-1,9]
	if(code < 0) {
	  if(code == -1) {
	    // -1
	    strcpy(message,"Out of memory while building graph");
	  } else {
	    // (..,-2]
	    strcpy(origmessage,"SGB panic code has illegal value");
	  }
	} else {
	  if(code == 0) {
	    // 0
	    strcpy(origmessage,"There seems to be an SGB error, but there's no panic code set");
	  } else {
	    // [1,9]
	    strcpy(message,"Out of memory before building graph");
	  }
	}
      } else {
	// 10
	strcpy(message,"Problem opening .dat or .gb file");
      }
    } else {
      // [11,29]
      if(code == 11) {
	//11
	strcpy(message,"Problem closing .dat or .gb file");
      } else{
	// [12,29]
	strcpy(message,"Problem parsing .dat or .gb file");
      }
    }
  } else {
    // [30,99]
    if(code < 60) {
    // [30,59]
      if(code < 50) {
	// [30,49]
	if(code < 40) {
	  // [30,39]
	  strcpy(message,"Parameter has illegal value");
	} else {
	// [40,49]
	strcpy(message,"Parameter has incredibly illegal value");
	}
      } else {
	// [50,59]
	strcpy(message,"Graph parameter is NULL");
      }
    } else {
      // [60,99]
      if(code < 70) {
	// [60,69]
	strcpy(message,"Bad graph or dist parameter");
      } else {
	if(code >= 90 && code < 99) {
	  // [90,99]
	  strcpy(message,"Internal consistency failure");
	} else {
	  // [70,89] [100,..)
	  strcpy(origmessage,"SGB panic code has illegal value");
	}
      }
    }
  }

  return message;

}
char * substitute_meta_characters(const char * string) {
  int i = 0,
      j = 0;
  int add = 0;
  char * newstring;
  
  if (string) {
    if((string[0] <= '9' && string[0] >= '0') || 
       (string[0] <= 'Z' && string[0] >= 'A')    ) add++;
//    while(string[i] != '\0') {
//      if(!((string[i] <= '9' && string[i] >= '0') ||
//	   (string[i] <= 'z' && string[i] >= 'a') ||
//	   (string[i] <= 'Z' && string[i] >= 'A') ||
//	   string[i] == '_')) add += 2;
//      i++;
//    }
    newstring = new char[strlen(string)+add+1];

    i = 0;

    if((string[0] <= '9' && string[0] >= '0') || 
       (string[0] <= 'Z' && string[0] >= 'A')    ) {
      newstring[j++] = 'n';
      newstring[j++] = string[i++];
    } else {
      newstring[j++] = string[i++];
    }
      
    while(string[i] != '\0') {
      if(!((string[i] <= '9' && string[i] >= '0') ||
	   (string[i] <= 'z' && string[i] >= 'a') ||
	   (string[i] <= 'Z' && string[i] >= 'A') ||
	   string[i] == '_')) {
	newstring[j++] = '_';
//	newstring[j++] = 'a'+(string[i]%26);
//	newstring[j++] = '_';
      } else {
	newstring[j++] = string[i];
      }
      i++;
    }
    newstring[j] = '\0';
    
  } else {
    newstring = NULL;
  }  
  return newstring;

}

void print_edges_directed(Graph *g) {
  
  Vertex *v;
  Arc *a;
  char *d1, *d2;

  for(v = g->vertices; v < g->vertices + g->n; v++) {
    for(a = v->arcs; a; a = a->next) {
      
      cout << "edge(" << (d1 = substitute_meta_characters(v->name))
	   << "," << (d2 = substitute_meta_characters(a->tip->name))
	   << ")." << endl;
      delete[] d1;
      delete[] d2;
    }
  }

}

void print_edges_undirected(Graph *g) {
  
  Vertex *v;
  Arc *a;
  char *d1, *d2;

  assert(g);

  for(v = g->vertices; v < g->vertices + g->n; v++) {
    for(a = v->arcs; a; a = a->next) {
      if(a->tip <= v) {
	cout << "edge(" << (d1 = substitute_meta_characters(v->name))
	     << "," << (d2 = substitute_meta_characters(a->tip->name))
	     << ")." << endl;
	delete[] d1;
	delete[] d2;
      }
    }
  }

}


SymbolTable symTable;
queue<long> *longqueue;
unsigned long distlength1, distlength2;

%}

%union {
	long longval;
	long *longptr;
	unsigned long ulongval;
	char *strval;
	Graph *graphval;
}

%token BOARD COMPLETE TRANSITIVE EMPTY CIRCUIT CYCLE
%token SIMPLEX
%token SUBSETS DISJOINT_SUBSETS PETERSEN
%token PERMS ALL_PERMS
%token PARTS ALL_PARTS
%token BINARY ALL_TREES

%token BI_COMPLETE WHEEL

%token COMPLEMENT GUNION INTERSECTION LINES PRODUCT INDUCED

%token BI_BOOK BOOK

%token BI_LISA PLANE_LISA

%token ECON

%token GAMES

%token MILES

%token PARTIAL_GATES PROD RISC

%token PLANE PLANE_MILES

%token RAMAN

%token RANDOM_BIGRAPH RANDOM_GRAPH

%token ROGET

%token WORDS

%token ASSIGN

%token <strval> NAME

%token INTNUM

%token <strval> STRING

%token PRINT PRINTU PRINTD

%token DEFPRINT

%token VERTEX

%token EDGE

%type <graphval> function standardfunction transformerfunction persistent_graphvalue nonpersistent_graphvalue

%type <longval> parlong producttype

%type <ulongval> parulong

%type <longptr> pardist distlist parwordweights

%start statements

%%
statements:
		/* empty */
	|	statement ';' statements
	;

statement:
		NAME ASSIGN function { 
		  char *errmsg;
		  if ($3 == NULL) {
		    gbdlyyerror(errmsg=interpret_panic_code(panic_code));
		    delete[] errmsg;
		  }

		  SymIter iter = symTable.begin();

		  if ((iter = symTable.find($1)) != symTable.end()) {
		    gb_recycle((*iter).second); 
		    symTable.erase(iter);
		  }  
		  symTable.insert(StringGraphPair($1,$3)); 
		}
	|	printstatement
	;

printstatement: 
	        PRINT function {  
                  char *errmsg;
		  if ($2 == NULL) {
			gbdlyyerror(errmsg=interpret_panic_code(panic_code));
                        delete[] errmsg;
		  }
		  print_edges_undirected($2); 
		  gb_recycle($2); 
		} 	
	|	PRINTD function {  
		  char *errmsg;
		  if ($2 == NULL) {
			gbdlyyerror(errmsg=interpret_panic_code(panic_code));
                        delete[] errmsg;
		  }
		  print_edges_directed($2); 
		  gb_recycle($2); 
		} 	
	|	PRINTU function {  
		  char *errmsg;
		  if ($2 == NULL) {
			gbdlyyerror(errmsg=interpret_panic_code(panic_code));
                        delete[] errmsg;
		  }
		  print_edges_undirected($2); 
		  gb_recycle($2); 
		} 	
	|	PRINT NAME { 
		  SymIter iter = symTable.begin(); 
		  if ((iter = symTable.find($2)) != symTable.end()) {
		    if ((*iter).second) 
		      print_edges_undirected((*iter).second); 
		    else {
		      cerr << "%Trouble!! The looked up graph" 
			   << "doesn't seem to exist!" << endl;
		    } 
		  }
		  else cerr << "%Undeclared identifier: " << $2 << endl; 
		}
	|	DEFPRINT '{' printredefinition '}'
	;

function:
		standardfunction { $$ = $1; }
	|	transformerfunction { $$ = $1; }
	;

standardfunction:
		BOARD '(' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong 
			')' { $$ = board($3,$5,$7,$9,$11,$13,$15); }
	|	COMPLETE '(' 
			parlong 
			')' { $$ = complete($3); }
	|	TRANSITIVE '(' 
			parlong 
			')' { $$ = transitive($3); }
	|	EMPTY '('
			parlong 
			')' { $$ = empty($3); }
	|	CIRCUIT '(' 
			parlong 
			')' { $$ = circuit($3); }
	|	CYCLE '(' 
			parlong 
			')' { $$ = cycle($3); }

	|	SIMPLEX '(' 
			parulong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong 
			')' { $$ = simplex($3,$5,$7,$9,$11,$13,$15); }

	|	SUBSETS '(' 
			parulong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parulong ',' 
			parlong 
			')' { $$ = subsets($3,$5,$7,$9,$11,$13,$15,$17); }
	|	DISJOINT_SUBSETS '(' 
			parlong ',' 
			parulong 
			')' { $$ = disjoint_subsets($3,$5); }
	|	PETERSEN '(' ')' { $$ = petersen(); }

	|	PERMS '(' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parlong ',' 
			parulong ',' 
			parlong 
			')' { $$ = perms($3,$5,$7,$9,$11,$13,$15); }
	|	ALL_PERMS '(' 
			parlong ',' 
			parlong 
			')' { $$ = all_perms($3,$5); }

	|	PARTS '(' 
			parulong ',' 
			parulong ',' 
			parulong ',' 
			parlong 
			')' { $$ = parts($3,$5,$7,$9); }
	|	ALL_PARTS '(' 
			parulong ',' 
			parlong 
			')' { $$ = all_parts($3,$5); }

	|	BINARY '(' 
			parulong ',' 
			parulong ',' 
			parlong 
			')' { $$ = binary($3,$5,$7); }
	|	ALL_TREES '(' 
			parulong ',' 
			parlong 
			')' { $$ = all_trees($3,$5); }

	|	BI_COMPLETE '(' 
			parulong ',' 
			parulong ',' 
			parlong 
			')' { $$ = bi_complete($3,$5,$7); }
	|	WHEEL '(' 
			parulong ',' 
			parulong ',' 
			parlong 
			')' { $$ = wheel($3,$5,$7); }
	|	BI_BOOK '('
			STRING ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parlong ','
			parlong ','
			parlong
			')' { $$ = bi_book($3,$5,$7,$9,$11,$13,$15,$17); }
	|	BOOK '('
			STRING ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parlong ','
			parlong ','
			parlong
			')' { $$ = book($3,$5,$7,$9,$11,$13,$15,$17); }
	|	BI_LISA '('
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parlong
			')' { $$ = bi_lisa($3,$5,$7,$9,$11,$13,$15,$17); }
	|	PLANE_LISA '('
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong
			')' { $$ = plane_lisa($3,$5,$7,$9,$11,$13,$15,$17,$19); }
	|	ECON '('
			parulong ','
			parulong ','
			parulong ','
			parlong
			'(' { $$ = econ($3,$5,$7,$9); }
	|	GAMES '('
			parulong ','
			parlong ','
			parlong ','
			parlong ','
			parlong ','
			parlong ','
			parlong ','
			parlong
			')' { $$ = games($3,$5,$7,$9,$11,$13,$15,$17); }
	|	MILES '('
			parulong ','
			parlong ','
			parlong ','
			parlong ','
			parulong ','
			parulong ','
			parlong
			')' { $$ = miles($3,$5,$7,$9,$11,$13,$15); }
 /*	This one might prove problematic, because it gets one gate 
	graph as argument and destroys it in the process
	|	PARTIAL_GATES '('
 */

	|	PROD '('
			parulong ','
			parulong
			')' { $$ = prod($3,$5); }
	|	RISC '('
			parulong
			')' { $$ = risc($3); }
	|	PLANE '('
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parulong ','
			parlong
			')' { $$ = plane($3,$5,$7,$9,$11,$13); }
	|	PLANE_MILES '('
			parulong ','
			parlong ','
			parlong ','
			parlong ','
			parulong ','
			parulong ','
			parlong
			')' { $$ = plane_miles($3,$5,$7,$9,$11,$13,$15); }
	|	RAMAN '('
			parlong ','
			parlong ','
			parulong ','
			parulong
			')' { $$ = raman($3,$5,$7,$9); }
	|	RANDOM_BIGRAPH '('
			parulong ','
			parulong ','
			parulong ','
			parlong ','
			pardist ','
			pardist ','
			parlong ','
			parlong ','
			parlong
			')' { 
				if(distlength1 != $3) {
					cerr << "Distribution array length 1 doesn't match. Expected length: " << $3 << ", got: " << distlength1 << endl;
				}
				if(distlength2 != $5) {
					cerr << "Distribution array length 2 doesn't match. Expected length: " << $5 << ", got: " << distlength2 << endl;
				}
				$$ = random_bigraph($3,$5,$7,$9,$11,$13,$15,$17,$19); delete[] $11; delete[] $13; }
	|	RANDOM_GRAPH '('
			parulong ','
			parulong ','
			parlong ','
			parlong ','
			parlong ','
			pardist ','
			pardist ','
			parlong ','
			parlong ','
			parlong
			')' { 
                              if( $13 || $15 )
				{
                                  if(distlength1 != $3) 
                                    {
                                      cerr << "Distribution array length 1 "
                                           << "doesn't match. Expected "
                                           << "length: " << $3 << ", got: " 
                                           << distlength1 << endl;
                                    }
                                  if(distlength2 != $3) 
                                    {
                                      cerr << "Distribution array length 2 "
                                           << "doesn't match. Expected "
                                           << "length: " << $3 << ", got: " 
                                           << distlength2 << endl;
                                    }
				}
                              
                              $$ = random_graph($3,$5,$7,$9,$11,$13,$15,$17,$19,$21);
                              if($13 || $15) {delete[] $13; delete[] $15;}
                            }
	|	ROGET '('
			parulong ','
			parulong ','
			parulong ','
			parlong
			')' { $$ = roget($3,$5,$7,$9); }
	|	WORDS '('
			parulong ','
			parwordweights ','
			parlong ','
			parlong
			')' { $$ = words($3,$5,$7,$9); }
	;

transformerfunction:
		COMPLEMENT '(' 
			nonpersistent_graphvalue ',' 
			parlong ',' 
			parlong ',' 
			parlong 
			')' { $$ = complement($3,$5,$7,$9); gb_recycle($3); }
	|	COMPLEMENT '(' 
			persistent_graphvalue ',' 
			parlong ',' 
			parlong ',' 
			parlong 
			')' { $$ = complement($3,$5,$7,$9); }
	|	GUNION '(' 
			nonpersistent_graphvalue ',' 
			nonpersistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = gunion($3,$5,$7,$9); gb_recycle($3); gb_recycle($5); }
	|	GUNION '(' 
			nonpersistent_graphvalue ',' 
			persistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = gunion($3,$5,$7,$9); gb_recycle($3); }
	|	GUNION '(' 
			persistent_graphvalue ',' 
			nonpersistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = gunion($3,$5,$7,$9); gb_recycle($5); }
	|	GUNION '(' 
			persistent_graphvalue ',' 
			persistent_graphvalue ',' 
			parlong ',' 
			parlong ')' { $$ = gunion($3,$5,$7,$9); }
	|	INTERSECTION '(' 
			nonpersistent_graphvalue ',' 
			nonpersistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = intersection($3,$5,$7,$9); gb_recycle($3); gb_recycle($5); }
	|	INTERSECTION '(' 
			nonpersistent_graphvalue ',' 
			persistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = intersection($3,$5,$7,$9); gb_recycle($3); }
	|	INTERSECTION '(' 
			persistent_graphvalue ',' 
			nonpersistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = intersection($3,$5,$7,$9); gb_recycle($5);}
	|	INTERSECTION '(' 
			persistent_graphvalue ',' 
			persistent_graphvalue ',' 
			parlong ',' 
			parlong 
			')' { $$ = intersection($3,$5,$7,$9); }
	|	LINES '(' 
			nonpersistent_graphvalue ',' 
			parlong 
			')' { $$ = lines($3,$5); gb_recycle($3); }
	|	LINES '(' 
			persistent_graphvalue ',' 
			parlong 
			')' { $$ = lines($3,$5); }
	|	PRODUCT '(' 
			nonpersistent_graphvalue ',' 
			nonpersistent_graphvalue ',' 
			producttype ',' 
			parlong 
			')' { $$ = product($3,$5,$7,$9); gb_recycle($3); gb_recycle($5); }
	|	PRODUCT '(' 
			nonpersistent_graphvalue ',' 
			persistent_graphvalue ',' 
			producttype ',' 
			parlong 
			')' { $$ = product($3,$5,$7,$9); gb_recycle($3); }
	|	PRODUCT '(' 
			persistent_graphvalue ',' 
			nonpersistent_graphvalue ',' 
			producttype ',' 
			parlong 
			')' { $$ = product($3,$5,$7,$9); gb_recycle($5); }
	|	PRODUCT '(' 
			persistent_graphvalue ',' 
			persistent_graphvalue ',' 
			producttype ',' 
			parlong 
			')' { $$ = product($3,$5,$7,$9); }
	|	INDUCED '(' 
			nonpersistent_graphvalue ',' 
			STRING ',' 
			parlong ',' 
			parlong ',' 
			parlong 
			')' { $$ = induced($3,$<strval>5,$7,$9,$11); gb_recycle($3); }
	|	INDUCED '(' 
			persistent_graphvalue ',' 
			STRING ',' 
			parlong ',' 
			parlong ',' 
			parlong 
			')' { $$ = induced($3,$<strval>5,$7,$9,$11); }
	;

parlong:
		INTNUM { $$ = $<longval>1; }
	|	'+' INTNUM { $$ = $<longval>2; }
	|	'-' INTNUM { $$ = -$<longval>2; }
	;

parulong:
		INTNUM { $$ = $<longval>1; }
	|	'+' INTNUM { $$ = $<longval>2; }
	;

pardist:
		'{' distlist '}' { $$ = $2; }
	|        INTNUM { if( $<longval>1 ) yyerror("Distribution parameter (0 or {...}) expected.\n"); else $$=NULL; }  
	;

distlist:
		/* empty */ { $$ = NULL; }
	|	INTNUM { 
			longqueue = new queue<long>;
			longqueue->push($<longval>1); 
		} 
		distrestlist { 
	                int i = 0;
			long acc = 0;
			$$ = new long[longqueue->size()]; 
#undef empty
			for(;!(longqueue->empty()) && acc < 0x40000000;i++) {
			  $$[i] = longqueue->front();
			  longqueue->pop();
			}
			if(!(longqueue->empty()) || acc > 0x40000000) {
				delete longqueue;
				delete[] $$;
				yyerror("% Illegal distribution parameter, the numbers must sum up to 2^30 _exactly_!\n");
			}
			delete longqueue;
		}
	;

distrestlist:
		/* empty */
	|	',' INTNUM { longqueue->push($<longval>2); } distrestlist
	;

nonpersistent_graphvalue:
		function { $$ = $1; }
	;

parwordweights:
		'{' INTNUM ',' INTNUM ',' INTNUM ',' INTNUM ',' INTNUM ','
		INTNUM ',' INTNUM ',' INTNUM ',' INTNUM '}' {
		        $$ = new long[9]; 
			$$[0] = $<longval>2;
			$$[1] = $<longval>4;
			$$[2] = $<longval>6;
			$$[3] = $<longval>8;
			$$[4] = $<longval>10;
			$$[5] = $<longval>12;
			$$[6] = $<longval>14;
			$$[7] = $<longval>16;
			$$[8] = $<longval>18;
		}
        |
		INTNUM { if( $<longval>1 ) yyerror("Weightlist parameter (0 or {a,b,w1,w2,w3,w4,w5,w6,w7}) expected.\n"); else $$=NULL; }  
        ;

persistent_graphvalue:
		NAME { 
			SymIter iter = symTable.begin(); 
			if((iter = symTable.find($1)) == symTable.end()) {
			  $$ = NULL; 
			  cerr << "%Undeclared identifier: " << $1 << endl;
			} else {
			  $$ = (*iter).second; 
			} 
		}
	;

producttype:
		parlong { $$ = $1; }
	|	NAME { 
		  if (!strcmp($1,"cartesian"))
		    $$ = cartesian;
		  else if (!strcmp($1,"direct"))
		    $$ = direct;
		  else if (!strcmp($1,"strong"))
		    $$ = strong;
		  else gbdlyyerror("Syntax Error (expected cartesian, direct or strong)");
		}
	;

printredefinition:
		/* empty */
	|	printpair ';' printredefinition
	;

printpair:
		vertex_expression ':' STRING
	;

vertex_expression:
		vertex_atomic_expression
	|	vertex_atomic_expression ',' vertex_expression
	;

vertex_atomic_expression:
		VERTEX NAME
	|	EDGE '(' NAME ',' NAME ')'
	|	NAME '<' NAME {}
	;
		

%%
