%{
/* 
 * GBDL
 * 
 * Wolfgang Faber
 *
 * created: Oct 24 1997
 *
 */

/* Scanner
 */

#include "gbdl.h"
#include "gbdl_parser.h"

void gbdlyyerror(char *s);
extern int gbdlyydebug;

%}

A [Aa]
B [Bb]
C [Cc]
D [Dd]
E [Ee]
F [Ff]
G [Gg]
H [Hh]
I [Ii]
J [Jj]
K [Kk]
L [Ll]
M [Mm]
N [Nn]
O [Oo]
P [Pp]
Q [Qq]
R [Rr]
S [Ss]
T [Tt]
U [Uu]
V [Vv]
W [Ww]
X [Xx]
Y [Yy]
Z [Zz]

%%

    /* Keywords (from the Stanford GraphBase) */

    /* Standard graph generators (see GB_BASIC) */
 
{B}{O}{A}{R}{D}				{ return BOARD; }
{C}{O}{M}{P}{L}{E}{T}{E}		{ return COMPLETE; }
{T}{R}{A}{N}{S}{I}{T}{I}{V}{E}	{ return TRANSITIVE; }
{E}{M}{P}{T}{Y}				{ return EMPTY; }
{C}{I}{R}{C}{U}{I}{T}			{ return CIRCUIT; }
{C}{Y}{C}{L}{E}				{ return CYCLE; }

{S}{I}{M}{P}{L}{E}{X}			{ return SIMPLEX; }

{S}{U}{B}{S}{E}{T}{S}			{ return SUBSETS; }
{D}{I}{S}{J}{O}{I}{N}{T}_{S}{U}{B}{S}{E}{T}{S} { return DISJOINT_SUBSETS; }
{P}{E}{T}{E}{R}{S}{E}{N}		{ return PETERSEN; }

{P}{E}{R}{M}{S}				{ return PERMS; }
{A}{L}{L}_{P}{E}{R}{M}{S}		{ return ALL_PERMS; }

{P}{A}{R}{T}{S}				{ return PARTS; }
{A}{L}{L}_{P}{A}{R}{T}{S}		{ return ALL_PARTS; }

{B}{I}{N}{A}{R}{Y}				{ return BINARY; }
{A}{L}{L}_{T}{R}{E}{E}{S}		{ return ALL_TREES; }
 
    /* Standard applications of induced */
 
{B}{I}_{C}{O}{M}{P}{L}{E}{T}{E}	{ return BI_COMPLETE; }
{W}{H}{E}{E}{L}				{ return WHEEL; }
 
    /* Graph transformers (see GB_BASIC) */
 
{C}{O}{M}{P}{L}{E}{M}{E}{N}{T}	{ return COMPLEMENT; }
 
 /* Union instead of Gunion is also allowed */
{G}?{U}{N}{I}{O}{N}			{ return GUNION; }
{I}{N}{T}{E}{R}{S}{E}{C}{T}{I}{O}{N} { return INTERSECTION; }
{L}{I}{N}{E}{S}				{ return LINES; }
{P}{R}{O}{D}{U}{C}{T}			{ return PRODUCT; }
{I}{N}{D}{U}{C}{E}{D}			{ return INDUCED; }

    /* Book derived graphs (see GB_BOOK) */

{B}{I}_{B}{O}{O}{K}			{ return BI_BOOK; }
{B}{O}{O}{K}				{ return BOOK; }

    /* Mona Lisa derived graphs (see GB_LISA) */

{B}{I}_{L}{I}{S}{A}			{ return BI_LISA; }
{P}{L}{A}{N}{E}_{L}{I}{S}{A}		{ return PLANE_LISA; }

    /* Economic sectors derived graphs (see GB_ECON) */

{E}{C}{O}{N}				{ return ECON; }

    /* Football scores derived graphs (see GB_GAMES) */

{G}{A}{M}{E}{S}				{ return GAMES; }

    /* Distances between cities derived graphs (see GB_MILES) */

{M}{I}{L}{E}{S}				{ return MILES; }

    /* Gate graphs (see GB_GATES) */

{P}{A}{R}{T}{I}{A}{L}_{G}{A}{T}{E}{S}	{ return PARTIAL_GATES; }
{P}{R}{O}{D}				{ return PROD; }
{R}{I}{S}{C}				{ return RISC; }

    /* Planar graphs (see GB_PLANE) */

{P}{L}{A}{N}{E}				{ return PLANE; }
{P}{L}{A}{N}{E}_{M}{I}{L}{E}{S}		{ return PLANE_MILES; }

    /* Ramanujan graphs (see GB_RAMAN) */

{R}{A}{M}{A}{N}				{ return RAMAN; }

    /* 'Deterministic random' graphs (see GB_RAND) */

{R}{A}{N}{D}{O}{M}_{B}{I}{G}{R}{A}{P}{H} {return RANDOM_BIGRAPH; }
{R}{A}{N}{D}{O}{M}_{G}{R}{A}{P}{H}	{ return RANDOM_GRAPH; }

    /* Thesaurus graphs (see GB_ROGET) */

{R}{O}{G}{E}{T}				{ return ROGET; }

    /* Words graphs (see GB_WORDS) */

{W}{O}{R}{D}{S}				{ return WORDS; }

    /* IO */

{P}{R}{I}{N}{T}_{D}			{ return PRINTD; } 

{P}{R}{I}{N}{T}_{U}			{ return PRINTU; } 

{P}{R}{I}{N}{T}				{ return PRINT; } 

{D}{E}{F}{P}{R}{I}{N}{T}		{ return DEFPRINT; }

{V}{E}{R}{T}{E}{X}			{ return VERTEX; }

{E}{D}{G}{E}				{ return EDGE; }



    /*********************************************************/
 

"="		{ return ASSIGN; }

[-+(),;}{]	{ return yytext[0]; }
 
    /* Names */
 
[A-Za-z][A-Za-z0-9_]*	{ gbdlyytext[gbdlyyleng] = '\0'; gbdlyylval.strval = new char[gbdlyyleng+1]; strcpy(gbdlyylval.strval,gbdlyytext); return NAME; }
 
    /* Numbers */
 
[0-9]+		{ gbdlyytext[gbdlyyleng] = '\0'; gbdlyylval.longval = atol(gbdlyytext); return INTNUM; }
 
    /* Strings */

\"[^"]*\"	{ gbdlyytext[gbdlyyleng-1] = '\0'; gbdlyylval.strval = new char[gbdlyyleng-2]; strcpy(gbdlyylval.strval,&(gbdlyytext[1])); return STRING; }
    /* we don't want the "" contained in the string */ 

\"[^"\n]*$	{ gbdlyyerror("Unterminated string"); }
 
\n		{ lines_parsed++; }

[ \t\r]+	;	/* white space */

^#.*$		;	/* comment */

.		{ gbdlyyerror("Illegal character"); }

%%


int gbdlyywrap() {
	return 1;
}

