/* Problem generator for disjunctive logic programming minimal 3SAT.

   compile with

   g++ -o gen-3sat gen-3sat.C

   Note: add query or constraints (e.g. ":- a1. :- a2.") and run with

   dl -n=1 [-FB]
*/


#include<iostream>
#include<fstream>
#include<stdlib.h>
#include<sys/times.h>


void main(int argc, char **argv)
{
   unsigned atoms;
   unsigned clauses;
   double ratio = 4.258;
   unsigned querysize = 2;

   if(argc == 4)
   {
      ratio = atof(argv[2]);
      querysize = atoi(argv[3]);
   }
   else if(argc == 3)
   {
      ratio = atof(argv[2]);
   }
   else if(argc != 2)
   {
      cerr << "USAGE: " << argv[0] << " num-atoms [ratio [query_size]]" << endl;
      exit(1);
   }
   atoms = atoi(argv[1]);
   if(atoms < 3)
   {
      cerr << "USAGE: " << argv[0] << " num-atoms >= 3 [ratio [query_size]]"
           << endl;
      exit(1);
   }

   // alternatively, number of clauses could be an argument
   // clauses = atoi(argv[2]);

   // produce "hard" problems (w.r.t. normal NP-complete 3SAT)
   clauses = (unsigned)(((double)atoms) * ratio);

   tms tms;
   srandom(times(&tms));

   unsigned clause_atoms[3];
   bool atom_positive[3];


   ofstream o("m3s.prop");


   for(unsigned i = 0; i < clauses; i++)
   {
      do
      {
      for(unsigned j = 0; j < 3; j++)
      {
         atom_positive[j] = (random()%2 == 1);
         clause_atoms[j] = random()%atoms;
      }
      } while ((clause_atoms[0] == clause_atoms[1]) ||
               (clause_atoms[0] == clause_atoms[2]) ||
               (clause_atoms[1] == clause_atoms[2]));



      cout << "clause(";
      for(unsigned j = 0; j < 3; j++)
      {
         if(atom_positive[j])
            cout << "a" << clause_atoms[j];
         else
            cout << "false";

         cout << ", ";
      }
      for(unsigned j = 0; j < 3; j++)
      {
         if(atom_positive[j])
            cout << "true";
         else
            cout << "a" << clause_atoms[j];

         if(j < 2)
            cout << ", ";
      }
      cout << ")." << endl;


      bool first = true;

      for(unsigned j = 0; j < 3; j++)
      {
         if(atom_positive[j])
         {
            if(! first)
               o << " v ";
            first = false;
            o << "a" << clause_atoms[j];
         }
      }

      first = true;
      for(unsigned j = 0; j < 3; j++)
      {
         if(!atom_positive[j])
         {
            if(! first)
               o << ", ";
            else
               o << " :- ";

            first = false;
            o << "a" << clause_atoms[j];
         }
      }
      
      o << "." << endl;
   }

   for(unsigned i = 0; i < querysize; i++)
   {
      if(i > 0) o << ", ";
      if(i > 0) cout << ", ";

      o << "a" << i;
      cout << "true(a" << i << ")";
   }
   o << " ?" << endl;
   cout << " ?" << endl;
}


