/// Randomly generate STRATCOMP instances as described in our KR98 paper
// (eite-etal-98a):
//
// - Every company is controlled by a number of consortia (CONTROLLERS) which
//   is set via the command-line (CONTROLLED_PER_COMPANY).
// - Each of these consortia consists of CONTROLLERS many companies.
// - Every product is produced by PRODUCERS many companies.
//
// Written by Gerald Pfeifer.

unsigned CONTROLLERS = 4;
const unsigned PRODUCERS = 4;

//////////////////////////////////////////////////////////////////////////////

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream>

using namespace std;

int main(int argc, char *argv[])
    {
    if( argc < 4  || argc > 5 )
        {
        cerr << "usage: " << argv[0]
             << " #companies"
                " #controlled_by/company #products/#companies"
                " [#controllers/controlled_by]"
             << endl;
        return -1;
        } 

    srand(getpid());

    const unsigned n=atoi(argv[1]);
    const unsigned CONTROLLED_PER_COMPANY = atoi(argv[2]);
    const unsigned PRODUCTS_DIV_COMPANIES = atoi(argv[3]);

    if( argc > 4 )
        CONTROLLERS=atoi(argv[4]);

    for(unsigned i=1; i <= n; i++)
        {
        for(unsigned j=1; j <= CONTROLLED_PER_COMPANY; j++)
            {
            cout << "controlled_by(" << i << ", ";

            for(unsigned j=1; j <= CONTROLLERS; j++)
                {
                unsigned guess;
                do
                     guess= (rand() % n) + 1;
                while( guess == i );

                cout << guess;

                if( j < CONTROLLERS )
     	            cout << ",";
                }

            cout << ")." << endl;
            }

//      if( ( rand() % 100 ) < 25 )
//          cout << ":- not strategic(" << n << ")." << endl;
        }

    for(unsigned i=1; i <= n*PRODUCTS_DIV_COMPANIES; i++)
        {
        cout << "produced_by(p" << i << ", ";

        for(unsigned j=1; j <= PRODUCERS; j++)
            {
            cout << (rand() % n) + 1;

            if( j < PRODUCERS )
     	        cout << ",";
            }
            
        cout << ")." << endl;            
        }
    }
