/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/*
 * File:   AggregateAtomGreater.h
 * Author: Carmine Dodaro
 *
 * Created on 22 June 2012, 17.24
 */

#ifndef _AGGREGATE_ATOM_GREATER_H
#define	_AGGREGATE_ATOM_GREATER_H

#include <cassert>

#include "Aggregate.h"
#include "Atom.h"

class ModelGenerator;

class AggregateAtomGreater : public Atom
{
    public:
        
        inline AggregateAtomGreater( AtomId id, string atomName );        
        inline virtual const Aggregate* getAggregate() const;
        inline virtual Aggregate* getAggregate();
        virtual functionPointer getPropagationFunction();
        inline virtual unsigned int getPositiveOccurrencesAggregates() const;
        inline virtual unsigned int getNegativeOccurrencesAggregates() const;
        inline virtual void incrementNegativeOccurrencesForAggregates( unsigned int value );
        inline virtual void incrementPositiveOccurrencesForAggregates( unsigned int value );
        inline virtual bool isAggregate() const;
        inline virtual void setAggregate( Aggregate* );
        
    private:
        /*
        * This is a pointer to the aggregate.
        */
        Aggregate* aggregate;
        
        unsigned int countNegativeOccurrencesForAggregates;
        
        unsigned int countPositiveOccurrencesForAggregates;        
};

AggregateAtomGreater::AggregateAtomGreater(
    AtomId id, 
    string atomName ) : Atom( id, atomName )
{
}

const Aggregate* 
AggregateAtomGreater::getAggregate() const
{
    return aggregate;
}

Aggregate* 
AggregateAtomGreater::getAggregate()
{
    return aggregate;
}

unsigned int 
AggregateAtomGreater::getNegativeOccurrencesAggregates() const
{
    return countNegativeOccurrencesForAggregates;
}

unsigned int 
AggregateAtomGreater::getPositiveOccurrencesAggregates() const
{
    return countPositiveOccurrencesForAggregates;
}

void
AggregateAtomGreater::incrementNegativeOccurrencesForAggregates( 
    unsigned int value )
{
    countNegativeOccurrencesForAggregates += value;
}

void
AggregateAtomGreater::incrementPositiveOccurrencesForAggregates( 
    unsigned int value )
{
    countPositiveOccurrencesForAggregates += value;
}

bool
AggregateAtomGreater::isAggregate() const
{
    return true;
}

void 
AggregateAtomGreater::setAggregate( 
    Aggregate* a )
{
    aggregate = a;
}

#endif