/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include <string.h>

#include "Atom.h"
#include "ModelGenerator.h"
#include <boost/algorithm/string.hpp>
extern int COMPLETED;

ostream&
operator<<(
    ostream & out,
    const Atom & atom )
{
    if( atom.isHidden() )
    {
        if( atom.isAggregate() )
        {
            const AggregateAtomGreater& aggregateAtomGreater = *( static_cast< const AggregateAtomGreater* >( &atom ) );
            out << *( aggregateAtomGreater.getAggregate() );
        }            
        else
            out << "#hidden" << atom.id;
    }
    else
        out << atom.atomName;
}

Atom::Atom(
    AtomId idOfAtom,
    string a ) 
: 
    id( idOfAtom ),
    atomName( a ),
    choice( 0 ),
    countFoundingRules( 0 ),
    isAtomTrueSupported ( false ),
    recursionLevel( MAXVALUE ),
    supportCounter( 0 ),
    truthValue( UNDEFINED ),
    typeOfDerivation( 0 ),
    sourcePointer( NULL ),
    founded( 0 ),
    idOfDerivation( { NULL } )
{
    vector< string > splittedString;
    boost::split( splittedString, a, boost::is_any_of("(") );
    if( splittedString.size() > 0 )
    {
        if( wasp::options.toPrint.size() != 0 && wasp::options.toPrint.find( splittedString[ 0 ] ) == wasp::options.toPrint.end() )
        {
            atomName = "";
        }

        if( wasp::options.doNotPrint.find( splittedString[ 0 ] ) != wasp::options.doNotPrint.end() )
        {
            atomName = "";
        }
    }    
}

Atom::Atom(
    const Atom & )
{    
    assert( 0 );
}

functionPointer 
Atom::getPropagationFunction()
{    
    return &ModelGenerator::aggregatesPropagationNothing;
}

bool
Atom::isFalse() const
{
    return ModelGenerator::getCurrentInterpretation()->isFalse( id );
}

bool
Atom::isFalsePropagated() const
{
    return ModelGenerator::getCurrentInterpretation()->isFalsePropagated( id );
}

bool
Atom::isTrue() const
{
    return ModelGenerator::getCurrentInterpretation()->isTrue( id );
}

bool
Atom::isTruePropagated() const
{
    return ModelGenerator::getCurrentInterpretation()->isTruePropagated( id );
}

bool
Atom::isUndefined() const
{
    return ModelGenerator::getCurrentInterpretation()->isUndefined( id );
}