/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/*
 * File:   AtomInAnAggregateSet.h
 * Author: Carmine Dodaro
 *
 * Created on 11 July 2012, 17.45
 */

#ifndef _ATOM_IN_AN_AGGREGATE_SET_H
#define	_ATOM_IN_AN_AGGREGATE_SET_H

#include <cassert>

#include "Atom.h"

class ModelGenerator;
class Aggregate;

class AtomInAnAggregateSet : public Atom
{
    public:
        
        inline AtomInAnAggregateSet( AtomId id, string atomName );        
        virtual functionPointer getPropagationFunction();        
        
        inline vector< Aggregate* >& getAggregatesWhereAtomAppears();
        inline vector< unsigned int >& getPositionInAggregatesWhereAtomAppears();
        
        inline Aggregate* getAggregate( unsigned int pos );
        
        inline void addAggregate( Aggregate* aggr, unsigned int position );
        inline virtual void incrementNegativeOccurrencesForAggregates( unsigned int value );
        inline virtual void incrementPositiveOccurrencesForAggregates( unsigned int value );
        inline virtual unsigned int getPositiveOccurrencesAggregates() const;    
        inline virtual unsigned int getNegativeOccurrencesAggregates() const;
        
        
    private:
        /*
        * A list of aggregates where this atom appears.
        */
        vector< Aggregate* > aggregatesWhereAtomAppears;

        /*
        * A list of position in which aggregates where this atom appears.
        */
        vector< unsigned int > positionInAggregatesWhereAtomAppears;
        
        unsigned int countPositiveOccurrencesForAggregates;
        
        unsigned int countNegativeOccurrencesForAggregates;
};

AtomInAnAggregateSet::AtomInAnAggregateSet(
    AtomId id, 
    string atomName ) : Atom( id, atomName ), countPositiveOccurrencesForAggregates( 0 ), countNegativeOccurrencesForAggregates( 0 )
{
}

vector< Aggregate* >&
AtomInAnAggregateSet::getAggregatesWhereAtomAppears()
{
    return aggregatesWhereAtomAppears;
}

vector< unsigned int >&
AtomInAnAggregateSet::getPositionInAggregatesWhereAtomAppears()
{
    return positionInAggregatesWhereAtomAppears;
}

void
AtomInAnAggregateSet::addAggregate(
    Aggregate* aggr,
    unsigned int position )
{
    this->aggregatesWhereAtomAppears.push_back( aggr );
    this->positionInAggregatesWhereAtomAppears.push_back( position );
}

void
AtomInAnAggregateSet::incrementPositiveOccurrencesForAggregates( 
    unsigned int value )
{
    countPositiveOccurrencesForAggregates += value;
}

void
AtomInAnAggregateSet::incrementNegativeOccurrencesForAggregates( 
    unsigned int value )
{
    countNegativeOccurrencesForAggregates += value;
}

unsigned int 
AtomInAnAggregateSet::getPositiveOccurrencesAggregates() const
{
    return countPositiveOccurrencesForAggregates;
}
    
unsigned int 
AtomInAnAggregateSet::getNegativeOccurrencesAggregates() const
{
    return countNegativeOccurrencesForAggregates;
}

Aggregate* 
AtomInAnAggregateSet::getAggregate( 
    unsigned int pos )
{
    assert( pos < aggregatesWhereAtomAppears.size() );
    assert( aggregatesWhereAtomAppears[ pos ] != NULL );
    return aggregatesWhereAtomAppears[ pos ];
}

#endif