/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   Constants.h
 * Author: Carmine Dodaro
 *
 * Created on 13 October 2010, 12.46
 */

#ifndef _CONSTANTS_H
#define	_CONSTANTS_H

#include <iostream>
#include <climits>
#include <vector>
using namespace std;

/*
 * Wasp constants
 */
#define SEPARATOR " "
#define MAXVALUE UINT_MAX
#define MAXAGGREGATE INT_MAX
#define WASP_STRING "WASP 1.0[ BETA ]\n"
#define NOMODEL "INCOHERENT"
#define NOMODEL_COMPETITION_OUTPUT "INCONSISTENT"
#define FOUND_INCOHERENCE MAXVALUE
#define ANSWER "ANSWER"
#define ANSWER_THIRD_COMPETITION "ANSWER SET FOUND"
#define WEIGHT_LEVEL_WEAKCONSTRAINT_SEPARATOR "@"
#define COST "COST"
#define OPTIMUM "OPTIMUM"
#define QUERY_FALSE_OUTPUT "no."
#define QUERY_TRUE_OUTPUT "yes."

#define WATCH_CHANGED UINT_MAX
#define NOTHING_TO_DO ( WATCH_CHANGED - 1 )

/* 
 * Error messages
 */
#define ERRORHEURISTIC WASP_STRING "\nThis heuristic is not supported."
#define QUERYNOTSUPPORTED WASP_STRING "\nNon-ground queries are not supported."
#define ERRORPARSING WASP_STRING "\nError during parsing."
#define VERSION_NOT_SUPPORTED WASP_STRING "\nThis version is no longer supported."
#define GRINGO_NEG_LIT_NOT_SUPPORTED WASP_STRING "\nNegative literals in the aggregate are not supported."

/*
 * Numeric Format
 */
#define COMMENT_ID -1
#define VERSION_ID 0
#define RULE_ID 1
#define HEAD_ID 4
#define BODY_ID 5
#define COUNT_ID 10
#define SUM_ID 11
#define WEAK_CONSTRAINT_ID 15
#define ATOM_NO_AGGREGATE 0
#define ATOM_IN_AGGREGATE_VALUE 1
#define AGGREGATE_GREATER_VALUE 2
#define AGGREGATE_DOUBLEGUARDS_VALUE 3
#define ATOM_TABLE_ID -2
#define ATOM_TABLE_ID_WITH_AGGREGATES -5
#define GROUNDER_MESSAGE -3
#define ATOM_SIZE -4
#define HAS_QUERY -6
#define GROUNDER_ERROR 0
#define GROUNDER_OK 1
#define INPUT_TRUE_SUPPORTED_ATOMS 2
#define INPUT_TRUE_UNSUPPORTED_ATOMS 3
#define INPUT_FALSE_ATOMS 4
#define QUERY_FALSE 0
#define QUERY_TRUE 1
#define QUERY_BRAVE 0
#define QUERY_CAUTIOUS 1

/*
 * Gringo Numeric Format
 */
#define GRINGO_NORMAL_RULE_ID 1
#define GRINGO_CONSTRAINT_RULE_ID 2
#define GRINGO_CHOICE_RULE_ID 3
#define GRINGO_DISJUNCTIVE_RULE_ID 8
#define GRINGO_LINE_SEPARATOR 0
#define GRINGO_BPLUS "B+"
#define GRINGO_BMINUS "B-"

enum GringoAggregateType{ GRINGO_ATOM_NO_AGGREGATE = 0, GRINGO_ATOM_IN_AGGREGATE_VALUE, GRINGO_AGGREGATE_GREATER_VALUE, GRINGO_AGGREGATE_DOUBLEGUARDS_VALUE };

/*
 * Watches
 */
#define WATCHED_BY_LEFT_WATCH 1
#define WATCHED_BY_RIGHT_WATCH 0
#define WATCHED_BY_TOP_WATCH 1
#define WATCHED_BY_BOTTOM_WATCH 0
#define NOT_WATCHED -1

/*
 * Interpretation
 */
#define UNDEFINED 1
#define TRUE 2
#define FALSE 4
#define TRUE_UNDEFINED 3
#define FALSE_UNDEFINED 5
#define INTERPRETATION_MASK 7

/*
 * Reasons
 */
#define TYPEALL 1
#define TYPEVECTOR 2
#define TYPESUPPORT 3
#define TYPEAGGREGATE 4
#define TYPEDETERMINISTIC 5

/*
 * Heuristics
 */
#define BERKMIN_HEURISTIC 0
#define FIRST_UNDEFINED_HEURISTIC 1
#define LOOKAHEAD_HEURISTIC 2
#define LOOKAHEADBERKMIN_HEURISTIC 3
#define BERKMIN_MOMS 4
#define BERKMIN_TWOLIGHT 5
#define BERKMIN_TWO 6
#define RANDOM_HEURISTIC 7
#define PRELOOKAHEAD_HEURISTIC 8
#define PRELOOKAHEADLIGHT_HEURISTIC 9
#define MIXED_HEURISTIC 10
#define MIXED_HEURISTIC_PRELOOKAHEAD 11
#define MIXED_HEURISTIC_RESTARTLOOKAHEAD 12
#define BALANCED_MO 13
#define BERKMIN_RANDOM 14
#define BERKMIN_LIMITED_ACTIVITY 15

/*
 * Options
 */
#define DELETE_WHEN_LEARN_A_NEW_CONSTRAINT 0
#define DELETE_WHEN_MODEL_IS_FOUND 1
#define NO_DELETE 2
#define RESTART_BASED_POLICY 3

/*
 * Wasprules
 */
#define RULE_UNDEFINED 0
#define RULE_CAN_SUPPORT 1
#define RULE_SATISFIED_BY_BODY 3
#define RULE_SATISFIED 7

class Rule;

typedef unsigned int AtomId;
typedef vector< AtomId > InvolvedVector;
class WaspRule;
union ReasonPointer
{
    WaspRule* waspRule;
    InvolvedVector* involvedVector;
};

/*
 * New types
 */
typedef double Activity;
typedef unsigned int AggregateId;
typedef unsigned int AggregateType;
typedef unsigned int ComponentId;
typedef unsigned int Level;
typedef unsigned int RuleId;
typedef unsigned int TruthValue;
typedef unsigned int InvolvedType;
typedef ReasonPointer InvolvedPointer;
typedef unsigned int QueryType;
typedef unsigned int QueryValue;
typedef pair< AtomId, TruthValue > AtomTruthValue;
typedef pair< ComponentId, AtomId > ComponentAtom;
typedef pair< ComponentId, Rule* > ComponentRule;
typedef pair< unsigned int, int > InvolvedRules;

#endif	/* _CONSTANTS_H */