/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include "DecisionHeuristic.h"
#include "ModelGenerator.h"
#include "Interpretation.h"

DecisionHeuristic::~DecisionHeuristic()
{
}

bool
DecisionHeuristic::propagate()
{
    return modelGenerator->propagate();
}

void 
DecisionHeuristic::nextLevel()
{
    modelGenerator->nextLevel();
}

void
DecisionHeuristic::unrollOne()
{
    modelGenerator->unrollOne();
}

void 
DecisionHeuristic::pushAtomInStack( 
    Atom& atom, 
    TruthValue truthValue )
{
    modelGenerator->pushAtomInStack( atom, truthValue );
}

Atom&
DecisionHeuristic::getAtom(
    AtomId atomId )
{
    return modelGenerator->program.getAtom( atomId );
}

Atom* 
DecisionHeuristic::getAtomPointer( 
    AtomId atomId )
{
    return modelGenerator->program.getAtomPointer( atomId );
}

Stack< unsigned >&
DecisionHeuristic::getBacktrackingStack()
{
    return modelGenerator->backtrackingStack;
}

stack< AtomTruthValue >&
DecisionHeuristic::getPropagationStack()
{
    return modelGenerator->propagationStack;
}

Interpretation&
DecisionHeuristic::getInterpretation()
{
    return modelGenerator->getInterpretation();
}

Program&
DecisionHeuristic::getProgram()
{
    return modelGenerator->getProgram();
}

void
DecisionHeuristic::setCallComputeGUS(
    bool value )
{
    modelGenerator->callComputeGUS = value;
}

void
DecisionHeuristic::computeGUS()
{
    modelGenerator->computeGUS();
}

unsigned int
DecisionHeuristic::estimateShortWaspRulesPropagation(
    const Literal& literal ) const
{
    return modelGenerator->estimateShortWaspRulesPropagation( literal );
}

bool
DecisionHeuristic::conflictLiteralIsSetted()
{
    return modelGenerator->conflictLiteral.first != 0;    
}