/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   DecisionHeuristic.h
 * Author: Carmine Dodaro
 *
 * Created on 04 December 2012, 10.25
 */

#ifndef _DECISIONHEURISTIC_H
#define	_DECISIONHEURISTIC_H

#include <cassert>
#include <vector>
#include <stack>
#include "Constants.h"
#include "stack.h"
#include "Literal.h"
using namespace std;

class Atom;
class ModelGenerator;
class Interpretation;
class Program;


class DecisionHeuristic
{
    public:
        inline DecisionHeuristic( ModelGenerator* mg );
        virtual ~DecisionHeuristic();
        
        virtual void makeAChoice() = 0;
        virtual void onUIPComputationTraversal( AtomId atomId, bool isTrue ){}
        virtual void onLearnedWaspRule( AtomId atomId, bool isTrue ){}
        virtual void onReachedANumberOfConflicts(){}
        virtual bool onRestart( unsigned int numberOfRestarts ){ return true; }
        virtual bool onInit(){ return true; }
        virtual void onProgramReduction(){}
        
        inline AtomId getHeuristicAtom();
        inline TruthValue getHeuristicPolarity();        
        
    protected:        
        AtomId chosenAtom;
        TruthValue chosenPolarity;
        
        Atom& getAtom( AtomId atomId );
        Atom* getAtomPointer( AtomId atomId );
        Stack< unsigned int >& getBacktrackingStack();
        stack< AtomTruthValue >& getPropagationStack();
        Interpretation& getInterpretation();
        Program& getProgram();
        void nextLevel();
        bool propagate();  
        void pushAtomInStack( Atom& atom, TruthValue truthValue );
        void unrollOne();
        void setCallComputeGUS( bool value );
        void computeGUS();
        unsigned int estimateShortWaspRulesPropagation( const Literal& literal ) const;
        bool conflictLiteralIsSetted();        
        
    private:
        ModelGenerator* modelGenerator;
};

DecisionHeuristic::DecisionHeuristic(
    ModelGenerator* mg )
{
    modelGenerator = mg;     
}

AtomId 
DecisionHeuristic::getHeuristicAtom()
{    
    return chosenAtom;
}

TruthValue 
DecisionHeuristic::getHeuristicPolarity()
{
    return chosenPolarity;
}

#endif /* _DECISIONHEURISTIC_H */