/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   DependencyGraph.h
 * Author: Carmine Dodaro
 *
 * Created on 22 October 2010, 13.40
 */

#ifndef _DEPENDENCYGRAPH_H
#define	_DEPENDENCYGRAPH_H

#include <unordered_set>
#include <vector>
#include "Component.h"

using namespace std;

class DependencyGraph 
{

    public:
        DependencyGraph();    
        virtual ~DependencyGraph();

        /**
        * This function adds an edge from v1 to v2.
        * @param v1 the first node.
        * @param v2 the second node.
        */
        void addEdge( unsigned int v1, unsigned int v2 );

        /**
        * Returns if there is a self loop for a node.
        * @param node the node.
        * 
        * @return true if there is a self loop, false otherwise.
        */
        bool existSelfLoop( unsigned int node );

        /**
        * This function calculates the strong connected components.
        * 
        * @return an array of components.
        */
        Component* componentsStrongConnected();

        /**
        * This function returns the size of component array.
        * @return the size of components array.
        */
        inline unsigned int getComponentSize();

        /**
        * This function prints the adjacency list. Can be removed when testing ends.
        */
        void printAdjacencyList();
        
        void clear();

    private:
    
        /**
        * The array of strong connected components.
        */
        Component* components;

        /**
        * The size of component array.
        */
        unsigned int componentSize;

        unordered_set< unsigned int > selfLoopVertex;

        DependencyGraph( const DependencyGraph& orig );
};

unsigned int
DependencyGraph::getComponentSize()
{
    return componentSize;
}

#endif	/* _DEPENDENCYGRAPH_H */
