/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/*
 * File:   Help.h
 * Author: Carmine Dodaro
 *
 * Created on 14 December 2012, 17.05
 */
#ifndef _WASPHELP_H
#define	_WASPHELP_H

#include <iostream>
using namespace std;
#include "Constants.h"

class Help
{
    public:
        static void printHelp()
        {            
            string separator = "\n---------------------------\n";
    
            cout << WASP_STRING << endl;            
            
            cout << "--det                                  - Compute the deterministic consequences and exit." << endl;
            cout << "--wf                                   - Compute the well-founded model and exit." << endl;
            cout << "--gringo                               - Read gringo numeric format." << endl;            
//            cout << "--simplify-program                     - Simplify the input program after the propagation of the deterministic consequences." << endl;            
//            cout << "--subsumption                          - Perform subsumption." << endl;
//            cout << "--no-subsumption                       - Disable subsumption." << endl;            
            cout << separator << endl;
            cout << "Learning options                       " << endl << endl;            
            cout << "--max-learnedconstraints-size=<N>      - Drop learned constraints with more than N literals." << endl;            
            cout << "--restarts=<N>                         - Luby et al. sequence using N as number of conflicts." << endl;
            cout << "--geometric-restarts=<N>               - Geometric restarts." << endl;
            cout << "--delete=<N>                           - Delete learned constraints policy: default(delete after learning), 1(delete after a model is found), 2(no delete), 3(restart based)." << endl;
            cout << separator << endl;
            cout << "Model Checker options                  " << endl << endl;            
            cout << "--polarity=<N>                         - Minisat polarity mode: default(false), 1(true), 2(random)." << endl;            
            cout << "--partial-mc-off                       - Disable partial model checking." << endl;
            cout << separator << endl;
            cout << "Output options                         " << endl << endl;
            cout << "--printprogram                         - Print the input program and exit." << endl;            
            cout << "--neg-filter=<pred1,...,predN>         - Do not print predicates pred1,...,predN." << endl;
            cout << "--filter=<pred1,...,predN>             - Print predicates pred1,...,predN." << endl;            
            cout << "--silent                               - Print nothing." << endl;            
            cout << "--competition-output                   - Print models according to the fourth competition output." << endl;
            cout << "--third-competition-output             - Print models according to the third competition output." << endl;            
            #ifndef STATS_OFF
            cout << separator << endl;
            cout << "Statistics options                     " << endl << endl;            
            cout << "--stats=<N>                            - Print stats every N choices." << endl;
            cout << "--stats-exit=<N>                       - Exit after N choices and print stats." << endl;           
            #endif
            #ifndef TRACE_OFF
            cout << separator << endl;
            cout << "Trace options                          " << endl << endl;            
            cout << "--trace-mg=<N>                         - Trace the model generator. N represents the verbosity." << endl;
            cout << "--trace-mc=<N>                         - Trace the model checker. N represents the verbosity." << endl;
            cout << "--trace-us=<N>                         - Trace the unfounded set function. N represents the verbosity." << endl;
            #endif
            cout << separator << endl;
            cout << "Heuristics options                     " << endl << endl;
//            cout << "--heuristic-lookahead                  - Use lookahead heuristic." << endl;
//            cout << "--heuristic-lookahead-berkmin          - Use lookahead and berkmin heuristics together." << endl;
            cout << "--heuristic-berkmin                    - Berkmin-like with look ahead." << endl;
            cout << "--heuristic-berkmintwo                 - Berkmin with two function." << endl;
            cout << "--heuristic-berkmintwolight            - Berkmin-like with a light version of two function." << endl;
//            cout << "--heuristic-berkminmoms                - Berkmin-like with moms function." << endl;
//            cout << "--heuristic-berkminbalancedmo          - Berkmin-like with balanced most occurrences function." << endl;
            cout << "--heuristic-firstundefined             - Berkmin-like with first undefined heuristic." << endl;
            cout << "--heuristic-random=<N>                 - Use random heuristic. Optional N is the seed." << endl;
            cout << "--heuristic-prelookahead=<N>           - Use prelookahead for N iterations." << endl;
            cout << "--heuristic-prelookaheadlight=<N>      - Use a light version of prelookahead for N iterations." << endl;
            cout << "--heuristic-mixed                      - Use a mixed heuristic." << endl;
            cout << "--heuristic-mixedprelookahead          - Use a mixed heuristic with initial prelookahead step." << endl;
            cout << "--heuristic-mixedrestartlookahead=<N>  - Use a mixed heuristic with a lookahead after N restarts." << endl;
            cout << "--heuristic-berkminrandom              - Berkmin-like with random choice instead of maximum activity." << endl;
            cout << "--heuristic-berkminlimitedactivity     - Berkmin-like with limited number of choices with maximum activity." << endl;
            cout << "--no-heuristic-aggregates              - Disable the heuristic for the aggregates." << endl;            
        }
};

#endif