/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   InputHandler.h
 * Author: Carmine Dodaro
 *
 * Created on 8 October 2010, 16.29
 */


#ifndef _INPUTHANDLER_H
#define	_INPUTHANDLER_H

#include <iostream>
using namespace std;

#include "Constants.h"
#include "Program.h"
#include "ProgramBuilder.h"
#include "WaspProgramBuilder.h"

class Atom;

class InputHandler
{

    public:

        /**
        * This constructor copy the passed program into member field program.
        *
        * @param program The istream input.
        */
        inline InputHandler( Program* p );
        inline ~InputHandler();

        inline void setBuilder( ProgramBuilder* b );

        /**
        * This function read instruction from standard input and
        * build the program.
        */
        void build();
        
        void build( istream& );
    
    private:
        Program* program;
        ProgramBuilder* builder;
};

InputHandler::~InputHandler()
{
    if( builder != NULL )
        delete builder;
}

InputHandler::InputHandler( 
    Program* p ) : program( p ), builder( NULL ) 
{
}

void
InputHandler::setBuilder(
    ProgramBuilder* b )
{
    assert( b != NULL );
    builder = b;
}

#endif	/* _INPUTHANDLER_H */
