/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   LearnedWaspRule.h
 * Author: Carmine Dodaro
 *
 * Created on 7 September 2012, 15.09
 */
#ifndef _LEARNEDWASPRULE_H
#define	_LEARNEDWASPRULE_H

#include <iostream>
#include <cassert>
using namespace std;

#include "WaspRule.h"

class LearnedWaspRule : public WaspRule
{
    public:
        inline LearnedWaspRule( unsigned int size );
        
        inline LearnedWaspRule();
        
        inline LearnedWaspRule( unsigned int size, unsigned int fWatch, unsigned int sWatch );
        
        virtual inline ~LearnedWaspRule();                
        
        /**
        * Return the activity value.
        *
        * @return activity value.
        */
        inline Activity getActivity() const;

        /**
        * Return the activity address.
        *
        * @return activity address.
        */
        inline Activity& getActivityAddress();
        
        /**
        * This function increments the activity value.
        *
        * @param increment The increment value.
        *
        */
        inline void incrementActivity( Activity );

        virtual bool isLearned() const;
        
        virtual bool isLocked() const;
        
        virtual bool isLearnedFromModel() const;
        
        inline void setLocked();
        
        inline unsigned int& getLockedAddress();

    private:
        
        /**
        * The activity of this learned waspRule.
        * This number computes how a learned waspRule is used.
        */
        Activity activity;
        
        unsigned int locked;
};

LearnedWaspRule::~LearnedWaspRule()
{
}

LearnedWaspRule::LearnedWaspRule(
    unsigned int size = 1 ) : WaspRule( size, 0, 0, size - 1 )
{
    activity = 0;
    locked = 0;
}

LearnedWaspRule::LearnedWaspRule( 
    unsigned int size, 
    unsigned int fWatch, 
    unsigned int sWatch ) : WaspRule( size, 0, fWatch, sWatch )
{
    activity = 0;
    locked = 0;
}

Activity
LearnedWaspRule::getActivity() const
{
    return activity;
}

Activity&
LearnedWaspRule::getActivityAddress()
{
    return activity;
}

void
LearnedWaspRule::incrementActivity(
    Activity increment )
{
    activity += increment;
}

void
LearnedWaspRule::setLocked()
{
    locked = 1;
}

unsigned int&
LearnedWaspRule::getLockedAddress()
{
    return locked;
}

#endif	/* _LEARNEDWASPRULE_H */
