/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include "unordered_map.h"
#include "Program.h"

#include <iostream>
using namespace std;

class MostOccurrencesForAggregates
{
    private:
        unordered_map< int, unordered_map< int, unsigned int > > binCache;
        unordered_map< int, unordered_map< int, unsigned int > > binSumCache;

        inline int normalizeMaxGuard( int guard );
        inline int normalizeMinGuard( int guard );

        pair< bool, unsigned int > findInBinCache( unsigned int n, unsigned int k );
        void insertIntoBinCache( unsigned int n, unsigned int k, unsigned int bin );
        pair< bool, unsigned int > findInBinSumCache( unsigned int n, unsigned int k );
        void insertIntoBinSumCache( unsigned int n, unsigned int k, unsigned int bin );
        unsigned int binomialCoefficient( unsigned int n, unsigned int k );
        unsigned int sumOfBinomialCoefficients( int from, int toInclusive, int n );
        void computeCombinatorialHeursticForCount( int normalizedUpperGuard, int normalizedLowerGuard, unsigned int& aggSize, unsigned int& aggOcc, unsigned int& posConjOcc, unsigned int& negConjOcc );        

    public:
        void computeCounters( Program* p );
        MostOccurrencesForAggregates();
};

int 
MostOccurrencesForAggregates::normalizeMaxGuard(
    int guard )
{
    //The upper guard is inclusive.
    int normalizedGuard = guard + 1;	                                   

    // handle strange cases where the guards are maxint.
    if( guard > normalizedGuard )
        normalizedGuard = guard;

    return normalizedGuard;
}

int 
MostOccurrencesForAggregates::normalizeMinGuard(
    int guard )
{
    //The upper guard is inclusive.
    int normalizedGuard = guard - 1;	                                   

    // handle strange cases where the guards are maxint.
    if( guard < normalizedGuard )
        normalizedGuard = guard;

    return normalizedGuard;
}