/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   ProgramBuilder.h
 * Author: Carmine Dodaro
 *
 * Created on 03 January 2013, 16.34
 */

#ifndef PROGRAMBUILDER_H
#define PROGRAMBUILDER_H

#include <cassert>
#include <iostream>
using namespace std;

#include "Constants.h"

class Aggregate;
class Atom;
class WaspRule;
class ChoiceRule;
class WeakConstraint;

class ProgramBuilder
{
	public:
		virtual void addAtom( AtomId atomId, string atomName ) = 0;
        virtual void addAtomInAnAggregateSet( AtomId, string atomName ) = 0;
        virtual void addAggregateAtomGreater( AtomId, string atomName ) = 0;
        virtual void addAggregateAtomDoubleGuards( AtomId, string atomName ) = 0;
        
        virtual AtomId addAuxAtom() = 0;
        virtual AtomId addAuxAtomForWeakConstraint() = 0;
        virtual AtomId addAuxAggregateAtomGreater() = 0;  
		
		virtual void addHeadAtom( WaspRule*, AtomId ) = 0;
		virtual void addBodyLiteral( WaspRule*, AtomId, bool positive ) = 0;
        
        virtual void addBodyLiteralInConstraint( WaspRule*, AtomId, bool positive ) = 0;
        
        virtual void addHeadAtomInChoiceRule( ChoiceRule*, AtomId ) = 0;
		virtual void addBodyLiteralInChoiceRule( ChoiceRule*, AtomId, bool positive ) = 0;

		virtual void startProgram() = 0;		
		virtual void endProgram() = 0;

		virtual void addFalseAtom( AtomId ) = 0;
		virtual void addTrueUnsupportedAtom( AtomId ) = 0;
		virtual void addTrueSupportedAtom( AtomId ) = 0;
		
		virtual void readComment( string text ) = 0;
		
		virtual void startMessageFromGrounding( unsigned int exitCode ) = 0;
		virtual void endMessageFromGrounding() = 0;
        
        virtual void setMessageFromGrounding( unsigned int type, unsigned int value ) = 0;
        virtual void setErrorFromGrounding( unsigned int error ) = 0;

		virtual Aggregate* startAggregateCount( AtomId id, unsigned int minGuard, unsigned int maxGuard, unsigned int size ) = 0;
        virtual void endAggregateCount( Aggregate* ) = 0;

		virtual Aggregate* startAggregateSum(  AtomId id, unsigned int minGuard, unsigned int maxGuard, unsigned int size ) = 0;
		virtual void endAggregateSum( Aggregate* ) = 0;

        virtual void addAtomInTheAggregate( int id, int value, Aggregate* aggr, unsigned int position ) = 0;        
        
        virtual WeakConstraint* startWeakConstraint( unsigned int size, int weight, int level ) = 0;
        virtual void addAuxAtomInWeakConstraint( WeakConstraint* wc, AtomId atomId ) = 0;
        virtual void addLiteralInWeakConstraint( WeakConstraint* wc, AtomId atomId, bool positive ) = 0;
		virtual void endWeakConstraint( WeakConstraint* ) = 0;
        
		virtual WaspRule* startWaspRule( unsigned int size, unsigned int numberOfHeadAtoms ) = 0;
		virtual void endWaspRule( WaspRule* ) = 0;
        
        virtual ChoiceRule* startChoiceRule( unsigned int numberOfBodyLiterals, unsigned int numberOfHeadAtoms ) = 0;
		virtual void endChoiceRule( ChoiceRule* ) = 0;
        
        virtual void setQueryType( QueryType ) = 0;
};

#endif /* PROGRAMBUILDER_H */
