/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   RuleContainer.h
 * Author: Carmine Dodaro
 *
 * Created on 27 December 2012, 12.07
 */
#ifndef _RULECONTAINER_H
#define	_RULECONTAINER_H

#include <cassert>
#include <iostream>
#include <unordered_map>
#include <vector>

#include "Constants.h"
using namespace std;

template< class T >
class RuleContainer : private vector< T >
{
	public:
	    inline RuleContainer();
	
	    using vector< T >::size;
	    inline const T& operator[]( unsigned idx ) const;
	
		void remove( T element );
		inline void push_back( T element );
        inline unsigned int getPosition( T element );

	private:
	    RuleContainer( const RuleContainer& );
	    
	    RuleContainer& operator=( const RuleContainer& );
	
        inline bool exists( T element );
		void swap( T element1, T element2 );

		unordered_map< T, unsigned int > position;
};

template< class T >
RuleContainer< T >::RuleContainer()
{
}

template< class T >
RuleContainer< T >::RuleContainer(
    const RuleContainer& )
{
    assert( 0 );
}

template< class T >
const T& RuleContainer< T >::operator[](
    unsigned idx ) const
{
    assert( idx < size() );
    return vector< T >::operator[]( idx );
}
 
template< class T >
RuleContainer< T >& RuleContainer< T >::operator=(
    const RuleContainer< T >& )
{
    assert( 0 );
    return *this;
}

template< class T >
void 
RuleContainer< T >::push_back( 
    T element )
{
    assert( !exists( element ) );
    position[ element ] = this->size();
    vector< T >::push_back( element );
}

template< class T >
bool 
RuleContainer< T >::exists( 
    T element )
{
    return position.find( element ) != position.end();
}

template< class T >
unsigned int
RuleContainer< T >::getPosition(
    T element )
{
    assert( exists( element ) );
    
    return position[ element ];
}

template< class T >
void 
RuleContainer< T >::remove( 
    T element )
{
    assert( !this->empty() );
    assert( exists( element ) );
    assert( position[ element ] < size() );
    
    unsigned pos = position[ element ];
    T& last = this->back();
    
    this->at( pos ) = last;
    position[ last ] = pos;
    
    this->pop_back();
    position.erase( element );
}
    
template< class T >
void 
RuleContainer< T >::swap( 
    T element1, 
    T element2 )
{
    unsigned int& pos1 = position[ element1 ];
    unsigned int& pos2 = position[ element2 ];

    T tmp = this->at( pos1 );
    this->at( pos1 ) = this->at( pos2 );
    this->at( pos2 ) = tmp;

    unsigned int tmp2 = pos1;
    pos1 = pos2;
    pos2 = tmp2;
}

#endif	/* _RULECONTAINER_H */
