/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   Statistics.cpp
 * Author: Carmine Dodaro
 * 
 * Created on 3 febbraio 2011, 12.36
 */

#include "Statistics.h"

#ifndef STATS_OFF

#include "timer.h"
using namespace wasp;

Statistics Statistics::statistics;

Statistics::Statistics():
    numModels( 0 ), numPropagations( 0 ), numComputeGUS( 0 ), numDeletedRules( 0 ), numCheckModelFailures( 0 ), rulesNumber( 0 ),
    atomsNumber( 0 ), initialPropagationNumber( 0 ), berkminInvocationsNumber( 0 ), higherGlobalCounterInvocationsNumber( 0 ), 
    learnRuleInvocationsNumber( 0 ), calculateFirstUipInvocationsNumber( 0 ), deleteLearnedRulesInvocationsNumber( 0 ), restartsNumber( 0 ), numberOfDerivationsGUS( 0 ), 
    numberOfNoDerivationsGUS( 0 ), gusInconsistences( 0 ), minConstraintSize( -1 ), maxConstraintSize( 0 ),
    countUnitPropagation( 0 ), countLastSupport( 0 ), countSupport( 0 ), countSubsumption( 0 ), countSelfSubsumption( 0 ), binaryRules( 0 ), ternaryRules( 0 ),        
    numberOfHornRules( 0 ), numberOfDisjunctiveFacts( 0 ), numberOfDisjunctiveRules( 0 ), numberOfConstraints( 0 ), numberOfLearnedWaspRules( 0 ),
    sizeOfLearnedWaspRules( 0 ), mostOccurrencesInvocationsNumber( 0 ), choosePolarityInvocationsNumber( 0 ), rulesNumberAfterReduction( 0 ), numberOfPartialModelChecks( 0 ),
    operatorRCalls( 0 ), fixPointOperatorRCalls( 0 ), numberOfChoices( 0 )
{
}

Statistics::Statistics(
    const Statistics& )
{
    assert( 0 );
}

Statistics::~Statistics()
{
}

void
Statistics::printStatistics()
{
    string separator = "\n---------------------------\n";
    
    cerr << "Initial Program Stats" << endl << endl;

    cerr << "Rules                          : " << rulesNumber << endl;
    cerr << "Atoms                          : " << atomsNumber << endl;
    cerr << "Deterministic Consequences     : " << initialPropagationNumber << endl;
    cerr << "Rules after reduction          : " << rulesNumberAfterReduction << endl;
    
    cerr << separator << endl;
    
    cerr << "Complete Stats" << endl << endl;    
    
    cerr << "Number of binary rules         : " << binaryRules << endl;
    cerr << "Number of ternary rules        : " << ternaryRules << endl;
    cerr << "Number of horn rules           : " << numberOfHornRules << endl;
    cerr << "Number of disjunctive facts    : " << numberOfDisjunctiveFacts << endl;
    cerr << "Number of disjunctive rules    : " << numberOfDisjunctiveRules << endl;
    cerr << "Number of constraints          : " << numberOfConstraints << endl;
    
    cerr << "Recursive HCF components       : " << sizeOfComponents.size() << endl;    
    for( unsigned int i = 0; i < sizeOfComponents.size(); i++ )
    {
        cerr << "       Component - Size        : " << i << " - " << sizeOfComponents[ i ] << endl;
    }
    
    cerr << separator << endl;

    cerr << "Dynamic Stats" << endl << endl;
    cerr << "Choices                        : " << numberOfChoices << endl;
    cerr << "Models                         : " << numModels << endl;
    cerr << "Max Backtracking Level         : " << maxBackTrackingLevel << endl;
//    for( int i = 1; i < reachedLevel.size(); i++ )
//    {
//    cerr << "       Reached Level           : " << i << " ( " << reachedLevel[ i ] << " times )" << endl;
//    }
    
    cerr << separator << endl;
    
    cerr << "Model Checker Stats" << endl << endl;
    cerr << "Check Model Failures           : " << numCheckModelFailures << endl;
    cerr << "Partial Model Checks           : " << numberOfPartialModelChecks << endl;
    cerr << "Operator R calls               : " << operatorRCalls << endl;
    cerr << "Fix Point OperatorR calls      : " << fixPointOperatorRCalls << endl;

    cerr << separator << endl;

    cerr << "Learning stats" << endl << endl;
    cerr << "CalculateFirstUip Invocations  : " << calculateFirstUipInvocationsNumber << endl;
    cerr << "Learned WaspRules              : " << numberOfLearnedWaspRules << endl;
    double averageSize = ( numberOfLearnedWaspRules > 0 ? sizeOfLearnedWaspRules / numberOfLearnedWaspRules : 0 );
    cerr << "   Average Size                : " << averageSize << endl;
    cerr << "Min Learned Constraints size   : " << minConstraintSize << endl;
    cerr << "Max Learned Constraints size   : " << maxConstraintSize << endl;    
    cerr << "DeleteLearnedRules Invocations : " << deleteLearnedRulesInvocationsNumber << endl;
    cerr << "Deleted Rules                  : " << numDeletedRules << endl;
    
    cerr << "Restarts number                : " << restartsNumber << endl;
    cerr << "Number of subsumptions         : " << countSubsumption << endl;
    cerr << "Number of self subsumptions    : " << countSelfSubsumption << endl;

    cerr << separator << endl;

    cerr << "Propagation Stats" << endl << endl;
    cerr << "Propagations                   : " << numPropagations << endl;   
    cerr << "Unit propagation Number        : " << countUnitPropagation << endl;
    cerr << "Support Inference Number       : " << countSupport << endl;
    cerr << "Last support Inference Number  : " << countLastSupport << endl;
    cerr << "GUS Inference Number           : " << numberOfDerivationsGUS << endl;

    cerr << separator << endl;

    cerr << "Heuristics Stats" << endl << endl;

    cerr << "Berkmin                        : " << berkminInvocationsNumber << endl;
    cerr << "   Higher Global Counter       : " << higherGlobalCounterInvocationsNumber << endl;
    cerr << "       Most Occurrences        : " << mostOccurrencesInvocationsNumber << endl;
    cerr << "       Choose Polarity         : " << choosePolarityInvocationsNumber << endl;

    cerr << separator << endl;

    cerr << "GUS" << endl << endl;

    cerr << "ComputeGUS Invocations          : " << numComputeGUS << endl;
    cerr << "Number of Derivations           : " << numberOfDerivationsGUS << endl;
    cerr << "Times GUS derives nothing       : " << numberOfNoDerivationsGUS << endl;
    cerr << "GUS Inconsistences              : " << gusInconsistences << endl;

    cerr << separator << endl;

    cerr << "Timers Stats" << endl << endl;

    cerr << "Start:" << endl;
    printTimer( cerr, " Startup                        : ", startup, "\n" );
    printTimer( cerr, " Initial Propagation            : ", initialPropagation, "\n" );

    cerr << "Heuristics: " << endl;
    printTimer( cerr, " Berkmin                        : ", berkmin, "\n" );
    printTimer( cerr, " Divide counters                : ", divideCountersTimer, "\n" );
    printTimer( cerr, " HigherGlobalCounter            : ", higherGlobalCounterTimer, "\n" );
    printTimer( cerr, " IsPT function                  : ", isPTFunction, "\n");
    printTimer( cerr, " Calculate Two                  : ", calculateTwo, "\n" );
    printTimer( cerr, " Lookahead                      : ", lookahead, "\n" );
    printTimer( cerr, "     Compute UTs                : ", computeUTs, "\n" );
    printTimer( cerr, "     Look2                      : ", look2, "\n" );
    printTimer( cerr, "     Compute PT                 : ", pt, "\n" );
    printTimer( cerr, "     MO                         : ", mo, "\n" );
    printTimer( cerr, "     Chosen                     : ", chosen, "\n" );

    cerr << "Learning: " << endl;
    printTimer( cerr, " Calculate First Uip            : ", calculateFirstUip, "\n" );
    printTimer( cerr, " Learn Rule                     : ", learnRule, "\n" );
    printTimer( cerr, " Delete Rules                   : ", deleteRules, "\n" );
    printTimer( cerr, " Subsumption                    : ", subsumption,"\n" );
    printTimer( cerr, " Self subsumption               : ", selfsubsumption,"\n" );

    cerr << "Propagation: " << endl;
    printTimer( cerr, " Total                          : ", propagation, "\n" );
    printTimer( cerr, "   Unit propagation             : ", unitPropagationTimer, "\n" );
    printTimer( cerr, "     Input Program              : ", scanNormalWaspRulesTimer, "\n" );
    printTimer( cerr, "     Learned WaspRules          : ", scanLearnedWaspRulesTimer, "\n" );    
    printTimer( cerr, "   SupportControl               : ", supportControlTimer, "\n" );
	printTimer( cerr, "   Calculate GUS                : ", computeGUS, "\n" );
    printTimer( cerr, "     Initial                    : ", computeGUSInitial, "\n" );
    printTimer( cerr, "     Central                    : ", computeGUSCentral, "\n" );
    printTimer( cerr, "     Final                      : ", computeGUSFinal, "\n" );
    printTimer( cerr, " Model Checker                  : ", modelChecker, "\n" );
    printTimer( cerr, "     Polynomial Operator        : ", polynomialOperator, "\n" );
    printTimer( cerr, "     Build SAT formula          : ", buildFormula, "\n" );    
    printTimer( cerr, "     SAT Solver Invocation      : ", satSolverInvocation, "\n" );

	cerr << "Utility functions: " << endl;
    printTimer( cerr, " Unroll                         : ", unroll, "\n" );
    printTimer( cerr, " UnrollOne                      : ", unrollOne, "\n" );
    printTimer( cerr, " Update interpretation watches  : ", updateInterpretationWatches, "\n" );
    printTimer( cerr, " Update left watch WaspRule     : ", updateLeftWatchWaspRule, "\n" );    
    printTimer( cerr, " Update right watch WaspRule    : ", updateRightWatchWaspRule, "\n" );    

    cerr << endl;
}

#endif
