/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   WaspNumericFormat.h
 * Author: Carmine Dodaro
 *
 * Created on 4 January 2013, 17.31
 */


#ifndef _WASPNUMERICFORMAT_H
#define	_WASPNUMERICFORMAT_H

#include <iostream>
using namespace std;

#include "Constants.h"
#include "Program.h"
#include "ProgramBuilder.h"

class Atom;

class WaspNumericFormat
{

public:

    WaspNumericFormat( ProgramBuilder* p );
  
    /**
     * This function read instruction from standard input and
     * build the program.
     */
    void parse();

    /**
     * This function read instruction from input and
     * build the program.
     *
     * @param input The istream input.
     */
    void parse( istream& input );
    
private:

    void parse_2_0( istream& input );
    
    void readComment( istream & input );
    void readWeakConstraint( istream & input );
    void readWaspRule( istream & input );
    void readCount( istream & input );
    void readSum( istream & input );
    void readAtomTable( istream& input );
    void readAtomTableWithAggregates( istream& input );
    void readMessage( istream& input );
    void readQuery( istream& input );
    void readTrueSupportedAtoms( istream& input );
    void readTrueUnsupportedAtoms( istream& input );
    void readFalseAtoms( istream& input );
        
    ProgramBuilder* programBuilder;
};

#endif	/* _WASPNUMERICFORMAT_H */
