/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   WaspProgramBuilder.h
 * Author: Carmine Dodaro
 *
 * Created on 3 January 2013, 16.37
 */


#ifndef _WASPPROGRAMBUILDER_H
#define	_WASPPROGRAMBUILDER_H

#include <iostream>
using namespace std;

#include "Constants.h"
#include "Program.h"
#include "ProgramBuilder.h"

class Atom;
class WaspRule;
class WeakConstraint;

class WaspProgramBuilder : public ProgramBuilder
{

    public:
        /**
        * This constructor copy the passed program into member field program.
        *
        * @param program A pointer to the program.
        */
        inline WaspProgramBuilder( Program* p );

        virtual void addAtom( AtomId atomId, string atomName );
        virtual void addAtomInAnAggregateSet( AtomId, string atomName );
        virtual void addAggregateAtomGreater( AtomId, string atomName );
        virtual void addAggregateAtomDoubleGuards( AtomId, string atomName );
        
        virtual AtomId addAuxAtom();
        virtual AtomId addAuxAtomForWeakConstraint();
        virtual AtomId addAuxAggregateAtomGreater();        
        
        virtual void addHeadAtom( WaspRule*, AtomId );
        virtual void addBodyLiteral( WaspRule*, AtomId, bool positive );
        
        virtual void addBodyLiteralInConstraint( WaspRule*, AtomId, bool positive );

        virtual void startProgram();
        virtual void endProgram();

        virtual void addFalseAtom( AtomId );
        virtual void addTrueUnsupportedAtom( AtomId );
        virtual void addTrueSupportedAtom( AtomId );

        virtual void readComment( string text );

        virtual void startMessageFromGrounding( unsigned int exitCode );
        virtual void endMessageFromGrounding();
        
        virtual void setMessageFromGrounding( unsigned int type, unsigned int value );
        virtual void setErrorFromGrounding( unsigned int error );
        
        virtual Aggregate* startAggregateCount( AtomId id, unsigned int minGuard, unsigned int maxGuard, unsigned int size );
        virtual void endAggregateCount( Aggregate* );

        virtual Aggregate* startAggregateSum(  AtomId id, unsigned int minGuard, unsigned int maxGuard, unsigned int size );
        virtual void endAggregateSum( Aggregate* );

        virtual void addAtomInTheAggregate( int id, int value, Aggregate* aggr, unsigned int position );
        
        virtual WeakConstraint* startWeakConstraint( unsigned int size, int weight, int level );
        virtual void addAuxAtomInWeakConstraint( WeakConstraint* wc, AtomId atomId );
        virtual void addLiteralInWeakConstraint( WeakConstraint* wc, AtomId atomId, bool positive );
		virtual void endWeakConstraint( WeakConstraint* );
        
        virtual WaspRule* startWaspRule( unsigned int size, unsigned int numberOfHeadAtoms );
        virtual void endWaspRule( WaspRule* );
        
        virtual ChoiceRule* startChoiceRule( unsigned int numberOfBodyLiterals, unsigned int numberOfHeadAtoms );
		virtual void endChoiceRule( ChoiceRule* );
        
        virtual void addHeadAtomInChoiceRule( ChoiceRule*, AtomId );
		virtual void addBodyLiteralInChoiceRule( ChoiceRule*, AtomId, bool positive );
        
        virtual void setQueryType( QueryType );
    
    private:
        Program* program;
        vector< pair< Atom*, WeakConstraint* > > atomsInWeakConstraints;
};

WaspProgramBuilder::WaspProgramBuilder( 
    Program* p ) : program( p )
{
}

#endif	/* _WASPPROGRAMBUILDER_H */