/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   WaspRuleContainer.h
 * Author: Carmine Dodaro
 *
 * Created on 27 December 2012, 12.07
 */
#ifndef _WASPRULECONTAINER_H
#define	_WASPRULECONTAINER_H

#include <cassert>
#include <iostream>
#include <unordered_map>
#include <vector>

#include "Constants.h"
using namespace std;

class WaspRule;

class WaspRuleContainer : public vector< WaspRule* >
{
	public:
		void remove( WaspRule* element );
		inline void push_back( WaspRule* element );
        inline unsigned int getPosition( WaspRule* element );

	private:
        inline bool exists( WaspRule* element );
		void swap( WaspRule* element1, WaspRule* element2 );

		unordered_map< WaspRule*, unsigned int > position;
};

void 
WaspRuleContainer::push_back( 
    WaspRule* element )
{
    assert( !exists( element ) );
    position[ element ] = size();
    vector< WaspRule* >::push_back( element );
}

bool 
WaspRuleContainer::exists( 
    WaspRule* element )
{
    return position.find( element ) != position.end();
}

unsigned int
WaspRuleContainer::getPosition(
    WaspRule* element )
{
    assert( exists( element ) );
    
    return position[ element ];
}

#endif	/* _WASPRULECONTAINER_H */