/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   WeakConstraint.h
 * Author: Carmine Dodaro
 *
 * Created on 10 March 2013, 18.24
 */
#ifndef _WEAKCONSTRAINT_H
#define	_WEAKCONSTRAINT_H

#include <cassert>
#include <iostream>
using namespace std;

#include "Constants.h"
#include "ProgramBuilder.h"
#include "WaspRule.h"

class WeakConstraint : public WaspRule
{
    friend ostream& operator<<( ostream& o, WeakConstraint& w )
    {   
        unsigned int bodyLiterals = w.getNumberOfLiteralsInTheBody();
        
        assert( bodyLiterals > 0 );
        o << ":~ ";
        o << w.getBodyLiteral( 0 );
            
        for( unsigned int i = 1; i < bodyLiterals; ++i )
        {
            o << ", " << w.getBodyLiteral( i );
        }
        o << ".";
        o << " [" << w.getWeight() << "@" << w.getLevel() << "]";
        
        return o;
    }
    
    public:        
        inline WeakConstraint( unsigned int size, int weight, int level );        
        inline ~WeakConstraint();
        
        inline int getWeight() const;
        inline int getLevel() const;
        inline int getLinearCost() const;
        inline void setLinearCost( int value );

//        inline bool isViolated() const;
        
    private:        
        inline WeakConstraint( const WeakConstraint& );
        int weight;
        int level;
        int linearCost;
};
WeakConstraint::WeakConstraint( 
     unsigned int s, 
     int w, 
     int l ) : WaspRule( s, 1, 0, s - 1 ), weight( w ), level( l ), linearCost( 0 )
 {
 }

WeakConstraint::WeakConstraint(
    const WeakConstraint& init ) : WaspRule( init )
{
    assert( 0 );
}

WeakConstraint::~WeakConstraint()
{
}

//bool
//WeakConstraint::isViolated() const
//{ 
//    return getLiteral( 0 ).getAtom()->isSupported(); 
//}

int
WeakConstraint::getLevel() const
{
    return level;
}

int
WeakConstraint::getWeight() const
{
    return weight;
}

int
WeakConstraint::getLinearCost() const
{
    return linearCost;
}

void
WeakConstraint::setLinearCost(
    int value )
{
    linearCost = value;
}
 

#endif	/* _WEAKCONSTRAINT_H */