/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include <vector>

#include "BerkminHeuristicBalancedMO.h"
#include "../Interpretation.h"
#include "../Program.h"
#include "../timer.h"

bool
BerkminHeuristicBalancedMO::pickLiteralWithActivity()
{
    if( higherGlobalCounterBalancedMO() )
    {
        choosePolarity( getAtom( chosenAtom ) );
        return true;
    }
    return false;    
}

bool
BerkminHeuristicBalancedMO::higherGlobalCounterBalancedMO()
{
    startTimer( higherGlobalCounterTimer );

    statistics( incrementHigherGlobalCounterInvocationsNumber() );
    HEURISTIC_TYPE max = 0;
    AtomId id = 0;    

    unsigned int mostOccurrencesMax = 0;
    AtomId mostOccurrencesId = 0;
    
    Interpretation& interpretation = getInterpretation();

    int i = interpretation.getLeftWatch();
    for( ; i <= interpretation.getRightWatch(); i++ )
    {
        if( interpretation.isUndefined( i ) )
        {
            if( getTotalCounter( i ) > max )
            {
                max = getTotalCounter( i );
                id = i;
                break;
            }            

            unsigned int negOccurrences = getNegativeOccurrences( i );
            unsigned int posOccurrences = getPositiveOccurrences( i );
            unsigned int value = ( ( negOccurrences * posOccurrences ) << 10 ) + ( negOccurrences + posOccurrences );
            
            if( value > mostOccurrencesMax )
            {
                mostOccurrencesMax = value;
                mostOccurrencesId = i;
            }
        }
    }
    
    i++;
    for( ; i <= interpretation.getRightWatch(); i++ )
    {
        if( interpretation.isUndefined( i ) )
        {
            if( getTotalCounter( i ) > max )
            {
                max = getTotalCounter( i );
                id = i;
            }            
        }
    }

    stopTimer( higherGlobalCounterTimer );

    if( id != 0 )
    {
        chosenAtom = id;
        return true;
    }
    else
    {
        assert( mostOccurrencesId != 0 );
        chosenAtom = mostOccurrencesId;
        return false;        
    }
}
