/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   BerkminHeuristicMoms.h
 * Author: Carmine Dodaro
 *
 * Created on 06 December 2012, 11.57
 */

#ifndef _BERKMINHEURISTICMOMS_H
#define	_BERKMINHEURISTICMOMS_H

#include <cassert>
#include "../Constants.h"
#include "BerkminHeuristic.h"

class BerkminHeuristicMoms : public BerkminHeuristic
{
    public:
        inline BerkminHeuristicMoms( ModelGenerator* mg, unsigned int atomsSize );
        
        virtual void makeAChoice();
        
    protected:
        virtual bool pickLiteralWithActivity();
        virtual bool tieBreak( vector< AtomId >& );
        virtual void choosePolarity( Atom& a );
        unsigned int calculateMoms( const Literal& l );
        virtual unsigned int higherGlobalCounterRandom();
        
        virtual bool onRestart( unsigned int numberOfRestarts );
        
    private:
        unsigned int atomsSize;
        unsigned int countMoms;
        inline bool checkAtomId( AtomId atomId ) const;
};

BerkminHeuristicMoms::BerkminHeuristicMoms(
    ModelGenerator* mg, 
    unsigned int atomsSize ) : BerkminHeuristic( mg, atomsSize ), countMoms( 0 )
{
    srand( options.momsHeuristicSeed );
}

#endif /* _BERKMINHEURISTICMOMS_H */