/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   BerkminHeuristicTwo.h
 * Author: Carmine Dodaro
 *
 * Created on 05 December 2012, 12.45
 */

#ifndef _BERKMINHEURISTICTWO_H
#define	_BERKMINHEURISTICTWO_H

#include <cassert>
#include "../Constants.h"
#include "BerkminHeuristic.h"
#include "../RuleContainer.h"
#include "../WatchedList.h"

class WaspRule;

class BerkminHeuristicTwo : public BerkminHeuristic
{
    public:
        BerkminHeuristicTwo( ModelGenerator* mg, unsigned int atomsSize );
        
    protected:
        
        vector< unsigned int > atomsTwoPositiveValues;
        vector< unsigned int > atomsTwoNegativeValues;
        
        virtual void choosePolarity( Atom& a );
        
        void calculateTwo( AtomId atomId );

        void calculateTwoFinal( AtomId atomId );
        
        void calculateTwoInit( AtomId atomId );
        
        void computeTwoValues( AtomId atomId, const RuleContainer< WaspRule* >& waspRules, bool positive );
        
        void computeTwoValues( AtomId atomId, const WatchedList< WaspRule* >& waspRules, bool positive );
        
        void computeTwoValuesFinal( AtomId atomId, const RuleContainer< WaspRule* >& waspRules, bool positive );
        
        void computeTwoValuesFinal( AtomId atomId, const WatchedList< WaspRule* >& waspRules, bool positive );    
        
        inline unsigned int getTwoNegative( AtomId atomId );
        
        inline unsigned int getTwoPositive( AtomId atomId );
                
        inline void incrementTwoNegative( AtomId atomId, unsigned int value );
        
        inline void incrementTwoPositive( AtomId atomId, unsigned int value ); 
        
        inline void incrementTwo( AtomId atomId, unsigned int value, bool positive );
        
        inline bool reachedMaxTwoValue( AtomId atomId, bool positive );
        
        inline void resetTwoCounters( AtomId atomId );
};

unsigned int 
BerkminHeuristicTwo::getTwoNegative( 
    AtomId atomId )
{
    assert( atomId > 0 && atomId < atomsTwoNegativeValues.size() );
    return atomsTwoNegativeValues[ atomId ];
}
        
unsigned int 
BerkminHeuristicTwo::getTwoPositive( 
    AtomId atomId )
{
    assert( atomId > 0 && atomId < atomsTwoPositiveValues.size() );
    return atomsTwoPositiveValues[ atomId ];
}

void 
BerkminHeuristicTwo::incrementTwoNegative( 
    AtomId atomId, 
    unsigned int value )
{
    assert( atomId > 0 && atomId < atomsTwoNegativeValues.size() );
    atomsTwoNegativeValues[ atomId ] += value;
}

void 
BerkminHeuristicTwo::incrementTwoPositive( 
    AtomId atomId, 
    unsigned int value )
{
    assert( atomId > 0 && atomId < atomsTwoPositiveValues.size() );
    atomsTwoPositiveValues[ atomId ] += value;
}

void 
BerkminHeuristicTwo::incrementTwo(
    AtomId atomId, 
    unsigned int value,
    bool positive )
{
    if( positive )
        incrementTwoPositive( atomId, value );
    else
        incrementTwoNegative( atomId, value );
}

bool 
BerkminHeuristicTwo::reachedMaxTwoValue( 
    AtomId atomId, 
    bool positive )
{
    if( positive )
        return getTwoPositive( atomId ) > 100;
    else
        return getTwoNegative( atomId ) > 100;
}

void 
BerkminHeuristicTwo::resetTwoCounters( 
    AtomId atomId )
{
    assert( atomId > 0 && atomId < atomsTwoPositiveValues.size() );
    assert( atomId > 0 && atomId < atomsTwoNegativeValues.size() );
    atomsTwoPositiveValues[ atomId ] = atomsTwoNegativeValues[ atomId ] = 0;
}

#endif /* _BERKMINHEURISTICTWO_H */
