/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include "MixedHeuristic.h"
#include "../Atom.h"
#include "../Interpretation.h"
#include "../Program.h"

#define MIXED_HEURISTIC_NUMBER 4

MixedHeuristic::~MixedHeuristic()
{
    for( int i = 0; i < heuristics.size(); i++ )
    {
        delete heuristics[ i ];
    }
}

bool
MixedHeuristic::onRestart(
    unsigned int numberOfRestarts )
{
    if( numberOfRestarts % MIXED_HEURISTIC_NUMBER == 0 )
    {
        index = ( index + 1 ) % heuristics.size();
    }
    
    return true;
}

void
MixedHeuristic::makeAChoice()
{
    assert( index < heuristics.size() );
    heuristics[ index ]->makeAChoice();
    
    chosenAtom = heuristics[ index ]->getHeuristicAtom();
    chosenPolarity = heuristics[ index ]->getHeuristicPolarity();   
}

void 
MixedHeuristic::onUIPComputationTraversal( 
    AtomId atomId, 
    bool isTrue )
{
    for( int i = 0; i < heuristics.size(); i++ )
    {
        heuristics[ i ]->onUIPComputationTraversal( atomId, isTrue );
    }
}

void
MixedHeuristic::onLearnedWaspRule( 
    AtomId atomId, 
    bool isTrue )
{
    for( int i = 0; i < heuristics.size(); i++ )
    {
        heuristics[ i ]->onLearnedWaspRule( atomId, isTrue );
    }
}

void 
MixedHeuristic::onReachedANumberOfConflicts()
{
    for( int i = 0; i < heuristics.size(); i++ )
    {
        heuristics[ i ]->onReachedANumberOfConflicts();
    }
}

bool
MixedHeuristic::onInit()
{
    return true;
}

void
MixedHeuristic::onProgramReduction()
{
    for( int i = 0; i < heuristics.size(); i++ )
    {
        heuristics[ i ]->onProgramReduction();
    }
}
