/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/*  
 * File:   MixedHeuristic.h
 * Author: Carmine Dodaro
 *
 * Created on 07 December 2012, 17.18
 */

#ifndef _MIXEDHEURISTIC_H
#define	_MIXEDHEURISTIC_H

#include <cassert>
#include "../Constants.h"
#include "FirstUndefinedHeuristic.h"
#include "BerkminHeuristic.h"
#include "PrelookaheadHeuristic.h"

class MixedHeuristic : public DecisionHeuristic
{
    public:
        inline MixedHeuristic( ModelGenerator* mg, DecisionHeuristic* h1, DecisionHeuristic* h2 );
        virtual ~MixedHeuristic();
        
        virtual void onUIPComputationTraversal( AtomId atomId, bool isTrue );
        virtual void onLearnedWaspRule( AtomId atomId, bool isTrue );
        virtual void onReachedANumberOfConflicts();        
        virtual void makeAChoice();
        virtual bool onRestart( unsigned int numberOfRestarts );
        virtual bool onInit();
        virtual void onProgramReduction();
        
        vector< DecisionHeuristic* > heuristics;
        unsigned int index;
};

MixedHeuristic::MixedHeuristic(
    ModelGenerator* mg,
    DecisionHeuristic* h1,
    DecisionHeuristic* h2 ) : DecisionHeuristic( mg )
{    
    heuristics.reserve( 2 );
    heuristics.push_back( h1 );
    heuristics.push_back( h2 );
    index = 0;
}

#endif /* _MIXEDHEURISTIC_H */