/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/*  
 * File:   MixedHeuristicPrelookahead.h
 * Author: Carmine Dodaro
 *
 * Created on 11 December 2012, 15.00
 */

#ifndef _MIXEDHEURISTICPRELOOKAHEAD_H
#define	_MIXEDHEURISTICPRELOOKAHEAD_H

#include <cassert>
#include "../Constants.h"
#include "MixedHeuristic.h"
#include "PrelookaheadHeuristic.h"
#include "../unordered_map.h"

class MixedHeuristicPrelookahead : public MixedHeuristic
{
    public:
        inline MixedHeuristicPrelookahead( ModelGenerator* mg, DecisionHeuristic* h1, DecisionHeuristic* h2 );
        virtual bool onInit();
        
    private:
        int performLookahead( AtomId atomId, Program& program, stack< AtomTruthValue >& propagationStack, Stack< unsigned int >& backtrackingStack );
        int performLookahead( AtomId atomId, TruthValue truthToSkip, Program& program, stack< AtomTruthValue >& propagationStack, Stack< unsigned int >& backtrackingStack );
        bool lookaheadInitial();
        
        void computeEquivalences( AtomId atomId );        
        unordered_map< AtomId, TruthValue > atomsToSkip;
};

MixedHeuristicPrelookahead::MixedHeuristicPrelookahead(
    ModelGenerator* mg, 
    DecisionHeuristic* h1, 
    DecisionHeuristic* h2 ) : MixedHeuristic( mg, h1, h2 )
{    
}

#endif /* _MIXEDHEURISTICPRELOOKAHEAD_H */