/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   PrelookaheadHeuristic.h
 * Author: Carmine Dodaro
 *
 * Created on 06 December 2012, 12.00
 */

#ifndef _PRELOOKAHEADHEURISTIC_H
#define	_PRELOOKAHEADHEURISTIC_H

#include <cassert>
#include <cstdlib>
#include "../Constants.h"
#include "BerkminHeuristic.h"

class ModelGenerator;

class PrelookaheadHeuristic : public BerkminHeuristic
{
    public:
        inline PrelookaheadHeuristic( ModelGenerator* mg, unsigned int atomsSize );        
        virtual bool onInit();
        
    protected:
        virtual void pickLiteralWithNoActivity();        
        virtual bool lookaheadInitial();        
        virtual int performLookahead( AtomId atomId, Program& program, stack< AtomTruthValue >& propagationStack, Stack< unsigned int >& backtrackingStack  );
        
        void doHeuristicPrelookahead();            
        vector< pair< AtomTruthValue, unsigned int > > preferredAtoms;
};

PrelookaheadHeuristic::PrelookaheadHeuristic( 
    ModelGenerator* mg, 
    unsigned int atomsSize ) : BerkminHeuristic( mg, atomsSize )
{
}

#endif /* _PRELOOKAHEADHEURISTIC_H */