/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include <vector>

#include "RandomHeuristic.h"
#include "../Interpretation.h"
#include "../Program.h"

void
RandomHeuristic::makeAChoice()
{
    Interpretation& interpretation = getInterpretation();
    
    if( !chooseWithCache( interpretation ) )
        chooseWithoutCache( interpretation );

    choosePolarity();
    undefinedAtoms.clear();
    assert( interpretation.isUndefined( chosenAtom ) );
}

void
RandomHeuristic::chooseWithoutCache(
    Interpretation& interpretation )
{    
    for( unsigned int i = interpretation.getLeftWatch(); i <= interpretation.getRightWatch(); i++ )
    {
        if( interpretation.isUndefined( i ) )
            undefinedAtoms.push_back( i );
    }
    
    unsigned int size = undefinedAtoms.size();    
    if( size > cacheSize )
    {                
        unsigned int i = 0;
        
        while( i < cacheSize )
        {
            unsigned int index = rand() % size;
            cacheAtoms.push_back( undefinedAtoms[ index ] );
            i++;
        }
    }
    
    unsigned int index = rand() % size;    
    chosenAtom = undefinedAtoms[ index ];
}

bool
RandomHeuristic::chooseWithCache(
    Interpretation& interpretation )
{
    while( !cacheAtoms.empty() )
    {
        AtomId atomId = cacheAtoms.back();        
        cacheAtoms.pop_back();
        
        if( interpretation.isUndefined( atomId ) )
        {
            chosenAtom = atomId;
            return true;
        }        
    }

    return false;
}