/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

/* 
 * File:   RandomHeuristic.h
 * Author: Carmine Dodaro
 *
 * Created on 06 December 2012, 11.43
 */

#ifndef _RANDOMHEURISTIC_H
#define	_RANDOMHEURISTIC_H

#include <cassert>
#include "../Constants.h"
#include "../DecisionHeuristic.h"

class Interpretation;

class RandomHeuristic : public DecisionHeuristic
{
    public:
        inline RandomHeuristic( ModelGenerator* mg, unsigned int atomsSize );
        virtual void makeAChoice();
        
    protected:
        bool chooseWithCache( Interpretation& );
        void chooseWithoutCache( Interpretation& );
        inline void choosePolarity();
        
    private:
        vector< AtomId > undefinedAtoms;
        vector< AtomId > cacheAtoms;
        unsigned int cacheSize;
};

RandomHeuristic::RandomHeuristic(
    ModelGenerator* mg,
    unsigned int atomsSize ) : DecisionHeuristic( mg )
{
    undefinedAtoms.reserve( atomsSize );
    
    cacheSize = 5;
    cacheAtoms.reserve( cacheSize );
    srand( options.randomHeuristicSeed );
}

void
RandomHeuristic::choosePolarity()
{
    chosenPolarity = ( ( rand() % 2 ) == 0 ) ? FALSE : TRUE;
}

#endif /* _RANDOMHEURISTIC_H */