/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#ifndef OPTIONS_H
#define OPTIONS_H

#include <iostream>
#include <vector>
#include <unordered_set>
#include <string>

#include "trace.h"
using namespace std;
namespace wasp
{

/**
 * This structure contains a field for each option of wasp.
 *
 * There is also a global instance of the structure, named options; probably,
 * the only instance in the program.
 */
struct Options
{
#ifndef TRACE_OFF
    TraceLevels traceLevels;
#endif
    
    int polarity;

    int numModels;

    bool silentMode;

    bool onlyWellFounded;

    bool onlyDeterministicConsequences;

    bool stats;
    
    bool statsExit;

    int heuristic;

    int deleteMode;

    bool printProgram;

    unsigned int statsTimer;

    vector< const char* > inputFiles;

    unordered_set< string > doNotPrint;

    unordered_set< string > toPrint;

    bool partialMCoff;

    bool competitionOutput;
    
    bool thirdCompetitionOutput;

    int maxLearnedConstraintSize;

    unsigned int depthlookahead;

    bool doSubsumption;

    unsigned int restartsNumbers;
    
    unsigned int randomHeuristicSeed;
    
    unsigned int momsHeuristicSeed;
    
    unsigned int prelookaheadRestartsNumber;
    
    bool mostOccurrencesForAggregates;
	
    bool geometricRestarts;
    
    string modelSeparator;
    
    bool gringoNumericFormat;
    
    bool simplifyProgram;
    
    unsigned int maxBerkminNumber;

    void parse( int argc, char* const* argv );
};

extern struct Options options;

};

#endif
