/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#ifndef TIMER_OFF

#include "timer.h"
#include <iomanip>

namespace wasp
{

struct Timers waspTimers;

ostream&
operator<<(
    ostream& out,
    const Timer& timer )
{
    return out << setprecision( 6 ) << fixed << timer.getElapsedTime() << "s";
}

Timer::Timer()
    : elapsedTime( 0 ),
      startTime( -1 )
{
}

void
Timer::start()
{    
    assert( isStopped() );

    startTime = clock();
}

void
Timer::stop()
{
    assert( ! isStopped() );

    elapsedTime += clock() - startTime;
    startTime = -1;
}

void
Timer::reset()
{
    assert( isStopped() );

    elapsedTime = 0;
}

double
Timer::getElapsedTime() const
{
    assert( isStopped() );

    return static_cast<double>( elapsedTime ) / CLOCKS_PER_SEC;
}

bool
Timer::isStopped() const
{
    return startTime == -1;
}

};

#endif
