/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#ifndef UNORDERED_MAP_H
#define UNORDERED_MAP_H

#include <iostream>
using namespace std;
#define HAVE_GCC_VERSION(MAJOR, MINOR) \
  (__GNUC__ > (MAJOR) || (__GNUC__ == (MAJOR) && __GNUC_MINOR__ >= (MINOR)))

#if HAVE_GCC_VERSION(4,3)
    #include <unordered_map>
    struct IntegerPairHasher
    {
        size_t operator()(const pair<unsigned int, unsigned int> &t) const
        {
            size_t h = t.first * 53 + t.second;

            return h;
        }
    };
   
    class Rule;
    struct RuleIntegerPairHasher
    {
        size_t operator()(const pair< unsigned int, Rule* > &t) const
        {
            size_t tmp = ( size_t ) t.second;
            size_t h = t.first * 53 + tmp;

            return h;
        }
   };
#elif HAVE_GCC_VERSION(3,3)
    // Works with GCC 3.1 and later, but we want to require 3.3 or later.
    #include <ext/hash_map>
    #define unordered_map __gnu_cxx::hash_map
    
    // Add an hashfunction for string to let g++4.0 compile in darwin
    namespace __gnu_cxx 
    {
        template<> struct hash< std::string >
        {
            size_t operator()( const std::string& x ) const
            {
                return hash<const char*>()( x.c_str() );
            }
        };
    }

#elif HAVE_GCC_VERSION(2,95)
    #warning "Please do not use versions of GCC before 3.3."
    #include <map>
    #define unordered_map map
#else
    #error "I do not know about this compiler yet."
#endif

#endif
