/*
*
*  Copyright 2013 Mario Alviano, Carmine Dodaro, Wolfgang Faber, Nicola Leone, Francesco Ricca, and Marco Sirianni.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include "ModelGenerator.h"
#include "options.h"
#include "trace.h"

int COMPLETED = 0;
int EXIT_CODE = 0;
using namespace wasp;

int 
main(
	int argc, 
	char** argv )
{
    options.parse( argc, argv );

    trace( mg, 1, "Starting with Model Generator\n" );
    
    if( options.onlyDeterministicConsequences )
    {
        ModelGenerator m;
        if( m.initialPropagation() )
        {
            m.printInterpretationWithUndefined();
        }
    }
    else if( options.onlyWellFounded )
    {
        ModelGenerator m;        
        m.computeGUS();
        m.printInterpretationWithUndefined();
    }
    else
    {
        ModelGenerator m( options.heuristic );
        m.mg();
        
        if( options.statsExit )
            statistics( printStatistics() );
    }
    return EXIT_CODE;
}
